# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/hemisphere.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_HemisphereDialog(object):
    def setupUi(self, HemisphereDialog):
        HemisphereDialog.setObjectName("HemisphereDialog")
        HemisphereDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(HemisphereDialog.sizePolicy().hasHeightForWidth())
        HemisphereDialog.setSizePolicy(sizePolicy)
        HemisphereDialog.setMinimumSize(QtCore.QSize(800, 600))
        HemisphereDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        HemisphereDialog.setSizeIncrement(QtCore.QSize(10, 10))
        HemisphereDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        HemisphereDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(HemisphereDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.groupBox_3 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout.setContentsMargins(10, 10, 10, 10)
        self.gridLayout.setSpacing(10)
        self.gridLayout.setObjectName("gridLayout")
        self.checkShowMeridian = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowMeridian.setFont(font)
        self.checkShowMeridian.setObjectName("checkShowMeridian")
        self.gridLayout.addWidget(self.checkShowMeridian, 1, 0, 1, 1)
        self.checkShowCelestial = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowCelestial.setFont(font)
        self.checkShowCelestial.setObjectName("checkShowCelestial")
        self.gridLayout.addWidget(self.checkShowCelestial, 2, 0, 1, 1)
        self.checkUseHorizon = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkUseHorizon.setFont(font)
        self.checkUseHorizon.setChecked(True)
        self.checkUseHorizon.setObjectName("checkUseHorizon")
        self.gridLayout.addWidget(self.checkUseHorizon, 0, 1, 1, 1)
        self.checkShowAlignStar = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowAlignStar.setFont(font)
        self.checkShowAlignStar.setObjectName("checkShowAlignStar")
        self.gridLayout.addWidget(self.checkShowAlignStar, 0, 0, 1, 1)
        self.checkShowSlewPath = QtWidgets.QCheckBox(self.groupBox_3)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkShowSlewPath.setFont(font)
        self.checkShowSlewPath.setObjectName("checkShowSlewPath")
        self.gridLayout.addWidget(self.checkShowSlewPath, 3, 0, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_3)
        self.line_3 = QtWidgets.QFrame(HemisphereDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout.addWidget(self.line_3)
        self.groupBox_2 = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox_2.setObjectName("groupBox_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_2)
        self.gridLayout_2.setContentsMargins(10, 10, 10, 10)
        self.gridLayout_2.setSpacing(10)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.checkPolarAlignment = QtWidgets.QRadioButton(self.groupBox_2)
        self.checkPolarAlignment.setEnabled(True)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkPolarAlignment.setFont(font)
        self.checkPolarAlignment.setObjectName("checkPolarAlignment")
        self.gridLayout_2.addWidget(self.checkPolarAlignment, 3, 0, 1, 1)
        self.checkEditHorizonMask = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditHorizonMask.setFont(font)
        self.checkEditHorizonMask.setObjectName("checkEditHorizonMask")
        self.gridLayout_2.addWidget(self.checkEditHorizonMask, 2, 0, 1, 1)
        self.checkEditBuildPoints = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditBuildPoints.setFont(font)
        self.checkEditBuildPoints.setObjectName("checkEditBuildPoints")
        self.gridLayout_2.addWidget(self.checkEditBuildPoints, 1, 0, 1, 1)
        self.checkEditNone = QtWidgets.QRadioButton(self.groupBox_2)
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.checkEditNone.setFont(font)
        self.checkEditNone.setChecked(True)
        self.checkEditNone.setObjectName("checkEditNone")
        self.gridLayout_2.addWidget(self.checkEditNone, 0, 0, 1, 1)
        self.addPositionToHorizon = QtWidgets.QPushButton(self.groupBox_2)
        self.addPositionToHorizon.setEnabled(False)
        self.addPositionToHorizon.setMinimumSize(QtCore.QSize(90, 0))
        self.addPositionToHorizon.setObjectName("addPositionToHorizon")
        self.gridLayout_2.addWidget(self.addPositionToHorizon, 2, 1, 1, 1)
        self.horizontalLayout.addWidget(self.groupBox_2)
        self.line_4 = QtWidgets.QFrame(HemisphereDialog)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout.addWidget(self.line_4)
        self.groupBox = QtWidgets.QGroupBox(HemisphereDialog)
        self.groupBox.setEnabled(True)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_2.setContentsMargins(10, 10, 10, 10)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.showPolar = QtWidgets.QCheckBox(self.groupBox)
        self.showPolar.setEnabled(True)
        self.showPolar.setObjectName("showPolar")
        self.verticalLayout_2.addWidget(self.showPolar)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(-1, 4, -1, 4)
        self.gridLayout_3.setSpacing(4)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setMinimumSize(QtCore.QSize(60, 0))
        self.label.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.gridLayout_3.addWidget(self.label, 0, 0, 1, 1)
        self.altitude = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.altitude.sizePolicy().hasHeightForWidth())
        self.altitude.setSizePolicy(sizePolicy)
        self.altitude.setMaximumSize(QtCore.QSize(60, 16777215))
        self.altitude.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.altitude.setObjectName("altitude")
        self.gridLayout_3.addWidget(self.altitude, 0, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setMinimumSize(QtCore.QSize(60, 0))
        self.label_2.setMaximumSize(QtCore.QSize(16777215, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout_3.addWidget(self.label_2, 1, 0, 1, 1)
        self.azimuth = QtWidgets.QLineEdit(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.azimuth.sizePolicy().hasHeightForWidth())
        self.azimuth.setSizePolicy(sizePolicy)
        self.azimuth.setMaximumSize(QtCore.QSize(60, 16777215))
        self.azimuth.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.azimuth.setObjectName("azimuth")
        self.gridLayout_3.addWidget(self.azimuth, 1, 1, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout_3)
        self.horizontalLayout.addWidget(self.groupBox)
        spacerItem = QtWidgets.QSpacerItem(80, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.horizontalLayout.setStretch(5, 1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(HemisphereDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.graphics = QtWidgets.QHBoxLayout()
        self.graphics.setObjectName("graphics")
        self.hemisphere = QtWidgets.QWidget(HemisphereDialog)
        self.hemisphere.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphere.sizePolicy().hasHeightForWidth())
        self.hemisphere.setSizePolicy(sizePolicy)
        self.hemisphere.setMinimumSize(QtCore.QSize(0, 270))
        self.hemisphere.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.hemisphere.setSizeIncrement(QtCore.QSize(10, 10))
        self.hemisphere.setBaseSize(QtCore.QSize(10, 10))
        self.hemisphere.setAutoFillBackground(True)
        self.hemisphere.setStyleSheet("")
        self.hemisphere.setObjectName("hemisphere")
        self.hemisphereMove = QtWidgets.QWidget(self.hemisphere)
        self.hemisphereMove.setEnabled(True)
        self.hemisphereMove.setGeometry(QtCore.QRect(1, 0, 391, 423))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.hemisphereMove.sizePolicy().hasHeightForWidth())
        self.hemisphereMove.setSizePolicy(sizePolicy)
        self.hemisphereMove.setMinimumSize(QtCore.QSize(0, 270))
        self.hemisphereMove.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.hemisphereMove.setSizeIncrement(QtCore.QSize(10, 10))
        self.hemisphereMove.setBaseSize(QtCore.QSize(10, 10))
        self.hemisphereMove.setAutoFillBackground(True)
        self.hemisphereMove.setStyleSheet("")
        self.hemisphereMove.setObjectName("hemisphereMove")
        self.graphics.addWidget(self.hemisphere)
        self.polar = QtWidgets.QWidget(HemisphereDialog)
        self.polar.setObjectName("polar")
        self.polarMove = QtWidgets.QWidget(self.polar)
        self.polarMove.setEnabled(False)
        self.polarMove.setGeometry(QtCore.QRect(0, 0, 391, 423))
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.polarMove.sizePolicy().hasHeightForWidth())
        self.polarMove.setSizePolicy(sizePolicy)
        self.polarMove.setMinimumSize(QtCore.QSize(0, 270))
        self.polarMove.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.polarMove.setSizeIncrement(QtCore.QSize(10, 10))
        self.polarMove.setBaseSize(QtCore.QSize(10, 10))
        self.polarMove.setAutoFillBackground(True)
        self.polarMove.setStyleSheet("")
        self.polarMove.setObjectName("polarMove")
        self.graphics.addWidget(self.polar)
        self.graphics.setStretch(0, 1)
        self.graphics.setStretch(1, 1)
        self.verticalLayout.addLayout(self.graphics)

        self.retranslateUi(HemisphereDialog)
        QtCore.QMetaObject.connectSlotsByName(HemisphereDialog)

    def retranslateUi(self, HemisphereDialog):
        _translate = QtCore.QCoreApplication.translate
        HemisphereDialog.setWindowTitle(_translate("HemisphereDialog", "Hemisphere"))
        self.groupBox_3.setToolTip(_translate("HemisphereDialog", "Showing different elements on the hemisphere diagram."))
        self.groupBox_3.setTitle(_translate("HemisphereDialog", "Show Elements"))
        self.checkShowMeridian.setText(_translate("HemisphereDialog", "Meridian limits "))
        self.checkShowCelestial.setText(_translate("HemisphereDialog", "Celestial paths "))
        self.checkUseHorizon.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Use horizon mask file in hemisphere window.</p></body></html>"))
        self.checkUseHorizon.setText(_translate("HemisphereDialog", "Horizon mask "))
        self.checkShowAlignStar.setText(_translate("HemisphereDialog", "Align stars "))
        self.checkShowSlewPath.setText(_translate("HemisphereDialog", "Slew path "))
        self.groupBox_2.setTitle(_translate("HemisphereDialog", "Operation Mode"))
        self.checkPolarAlignment.setToolTip(_translate("HemisphereDialog", "Enabling the polar or ortho align procedure. For starting the procedure, enable align stars and choose a polar align star with left mouse click and a ortho align star with right mouse click."))
        self.checkPolarAlignment.setText(_translate("HemisphereDialog", "Align Polar/Ortho "))
        self.checkEditHorizonMask.setToolTip(_translate("HemisphereDialog", "Enabling the edit of the horizon points."))
        self.checkEditHorizonMask.setText(_translate("HemisphereDialog", "Edit Horizon Mask "))
        self.checkEditBuildPoints.setToolTip(_translate("HemisphereDialog", "Enabling the edit mode for model build points."))
        self.checkEditBuildPoints.setText(_translate("HemisphereDialog", "Edit Build Points"))
        self.checkEditNone.setToolTip(_translate("HemisphereDialog", "Switching the hemisphere window to normal working mode."))
        self.checkEditNone.setText(_translate("HemisphereDialog", "Normal"))
        self.addPositionToHorizon.setToolTip(_translate("HemisphereDialog", "<html><head/><body><p>Add actual telescope position (Alt/Az) as point to the horizon.</p></body></html>"))
        self.addPositionToHorizon.setText(_translate("HemisphereDialog", "Add actual pos"))
        self.groupBox.setTitle(_translate("HemisphereDialog", "Views / Coords"))
        self.showPolar.setText(_translate("HemisphereDialog", "Show Polar Diagram"))
        self.label.setText(_translate("HemisphereDialog", "Altitude:"))
        self.label_2.setText(_translate("HemisphereDialog", "Azimuth:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    HemisphereDialog = QtWidgets.QWidget()
    ui = Ui_HemisphereDialog()
    ui.setupUi(HemisphereDialog)
    HemisphereDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages

# local imports
from mountcontrol.firmware import Firmware
from mountcontrol.setting import Setting
from mountcontrol.obsSite import ObsSite
from mountcontrol.model import Model
from mountcontrol.satellite import Satellite
from mountcontrol.geometry import Geometry


__all__ = ['Mount',
           ]


class Mount(object):
    """
    The Mount class is the main interface for interacting with the mount
    computer. The user could:
        setup / change the interface to the mount
        start / stop cyclic tasks to poll data from mount
        send and get data from mount
        has signals for interfacing to external GUI's for data updates

        >>> Mount(
        >>>       host=host,
        >>>       MAC=MAC,
        >>>       pathToData=pathToData,
        >>>       verbose=verbose,
        >>>       )

    """

    __all__ = ['Mount',
               ]

    log = logging.getLogger(__name__)
    DEFAULT_PORT = 3492

    def __init__(self,
                 host=None,
                 MAC=None,
                 pathToData=None,
                 verbose=None,
                 ):

        self._host = None
        self.MAC = MAC
        self.pathToData = pathToData
        self.verbose = verbose

        self.firmware = Firmware(self.host)
        self.setting = Setting(self.host)
        self.satellite = Satellite(self.host)
        self.obsSite = ObsSite(self.host,
                               pathToData=self.pathToData,
                               verbose=self.verbose,
                               )
        self.geometry = Geometry(obsSite=self.obsSite)
        self.model = Model(self.host, self.obsSite)
        self.host = host

    @property
    def host(self):
        return self._host

    @host.setter
    def host(self, value):
        value = self.checkFormatHost(value)
        self._host = value
        self.firmware.host = value
        self.setting.host = value
        self.model.host = value
        self.obsSite.host = value
        self.satellite.host = value

    @property
    def MAC(self):
        return self._MAC

    @MAC.setter
    def MAC(self, value):
        value = self.checkFormatMAC(value)
        self._MAC = value

    def checkFormatHost(self, value):
        """
        checkFormatHost ensures that the host ip and port is in correct format
        to enable socket connection later on. if no port is given, the default
        port for the mount will be added automatically

        :param      value: host value
        :return:    host value as tuple including port
        """

        if not value:
            self.log.info('Wrong host value: {0}'.format(value))
            return None

        if not isinstance(value, (tuple, str)):
            self.log.info('Wrong host value: {0}'.format(value))
            return None

        if isinstance(value, str):
            value = (value, self.DEFAULT_PORT)

        return value

    def checkFormatMAC(self, value):
        """
        checkFormatMAC makes some checks to ensure that the format of the
        string is ok for WOL package.

        :param      value: string with mac address
        :return:    checked string in upper cases
        """

        if not value:
            self.log.info('wrong MAC value: {0}'.format(value))
            return None

        if not isinstance(value, str):
            self.log.info('wrong MAC value: {0}'.format(value))
            return None

        value = value.upper()
        value = value.replace('.', ':')
        value = value.split(':')
        if len(value) != 6:
            self.log.info('wrong MAC value: {0}'.format(value))
            return None

        for chunk in value:
            if len(chunk) != 2:
                self.log.info('wrong MAC value: {0}'.format(value))
                return None

            for char in chunk:
                if char not in ['0', '1', '2', '3', '4',
                                '5', '6', '7', '8', '9',
                                'A', 'B', 'C', 'D', 'E', 'F']:
                    self.log.info('wrong MAC value: {0}'.format(value))
                    return None

        value = '{0:2s}:{1:2s}:{2:2s}:{3:2s}:{4:2s}:{5:2s}'.format(*value)
        return value

    def resetData(self):
        """
        resetData deletes all data already stored in classes just by redefining
        the classes. it send as well a signal, when the data is cleared.

        :return: nothing
        """

        self.firmware = Firmware(self.host)
        self.setting = Setting(self.host)
        self.model = Model(self.host)
        self.satellite = Satellite(self.host)
        self.obsSite = ObsSite(self.host,
                               pathToData=self.pathToData,
                               verbose=self.verbose,
                               )
        self.geometry = Geometry(obsSite=self.obsSite)

    def calcTransformationMatrices(self):
        """
        :return: alt az
        """
        ha = self.obsSite.haJNowTarget
        dec = self.obsSite.decJNowTarget
        lat = self.obsSite.location.latitude
        pierside = self.obsSite.piersideTarget
        alt, az, x, y, z = self.geometry.calcTransformationMatrices(ha=ha,
                                                                    dec=dec,
                                                                    lat=lat,
                                                                    pierside=pierside)

        return alt, az, x, y, z



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages

# local import


class SettHorizon:
    """
    """

    def __init__(self):
        self.ui.saveHorizonMask.clicked.connect(self.saveHorizonMask)
        self.ui.saveHorizonMaskAs.clicked.connect(self.saveHorizonMaskAs)
        self.ui.loadHorizonMask.clicked.connect(self.loadHorizonMask)
        self.ui.clearHorizonMask.clicked.connect(self.clearHorizonMask)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.horizonFileName.setText(config.get('horizonFileName', ''))
        fileName = self.app.config['mainW'].get('horizonFileName')
        self.app.data.loadHorizonP(fileName=fileName)

        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['horizonFileName'] = self.ui.horizonFileName.text()
        return True

    def loadHorizonMask(self):
        """
        :return: success
        """
        folder = self.app.mwGlob['configDir']
        fileTypes = 'Horizon mask files (*.hpts);; CSV Files (*.csv);; MW3 Files (*.txt)'
        loadFilePath, fileName, ext = self.openFile(self,
                                                    'Open horizon mask file',
                                                    folder,
                                                    fileTypes)
        if not loadFilePath:
            return False

        suc = self.app.data.loadHorizonP(fileName=fileName, ext=ext)

        if suc:
            self.ui.horizonFileName.setText(fileName)
            self.app.message.emit(f'Horizon mask [{fileName}] loaded', 0)
            self.app.uiWindows['showHemisphereW']['classObj'].drawHemisphere()
            self.app.drawHorizonPoints.emit()

        else:
            self.app.message.emit(f'Horizon mask [{fileName}] cannot no be loaded', 2)

        self.app.redrawHemisphere.emit()

        return True

    def saveHorizonMask(self):
        """
        :return: success
        """
        fileName = self.ui.horizonFileName.text()
        if not fileName:
            self.app.message.emit('Horizon mask file name not given', 2)
            return False

        suc = self.app.data.saveHorizonP(fileName=fileName)
        if suc:
            self.app.message.emit(f'Horizon mask [{fileName}] saved', 0)

        else:
            self.app.message.emit(f'Horizon mask [{fileName}] cannot no be saved', 2)

        return True

    def saveHorizonMaskAs(self):
        """
        :return: success
        """
        folder = self.app.mwGlob['configDir']
        saveFilePath, fileName, ext = self.saveFile(self,
                                                    'Save horizon mask file',
                                                    folder,
                                                    'Horizon mask files (*.hpts)',
                                                    )
        if not saveFilePath:
            return False

        suc = self.app.data.saveHorizonP(fileName=fileName)
        if suc:
            self.ui.horizonFileName.setText(fileName)
            self.app.message.emit(f'Horizon mask [{fileName}] saved', 0)

        else:
            self.app.message.emit(f'Horizon mask [{fileName}] cannot no be saved', 2)

        return True

    def clearHorizonMask(self):
        """
        :return:
        """
        self.app.data.horizonP = []
        self.ui.horizonFileName.setText('')
        self.app.redrawHemisphere.emit()
        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import shutil
import os

# external packages
from sgp4.exporter import export_tle

# local import


class DataWriter:
    """

    """
    __all__ = ['DataWriter',
               ]

    log = logging.getLogger(__name__)

    def __init__(self, app):
        super().__init__()
        self.app = app

    def writeEarthRotationData(self, installPath=''):
        """
        :param installPath:
        :return:
        """
        sourceDir = self.app.mwGlob['dataDir'] + '/'
        destDir = installPath + '/'

        if not os.path.isfile(sourceDir + 'tai-utc.dat'):
            return False

        if not os.path.isfile(sourceDir + 'finals2000A.all'):
            return False

        if destDir != sourceDir:
            shutil.copy(sourceDir + 'tai-utc.dat', destDir + 'tai-utc.dat')

        with open(sourceDir + 'finals2000A.all', 'r') as infile:
            with open(destDir + 'finals.data', 'w') as outfile:
                for line in infile:
                    if 19 < int(line[0:2]) < 50:
                        outfile.writelines(line)

        return True

    @staticmethod
    def writeCometMPC(datas=None, installPath=''):
        """
        data format of json and file description in
        https://minorplanetcenter.net/Extended_Files/Extended_MPCORB_Data_Format_Manual.pdf

        :param datas:
        :param installPath:
        :return:
        """
        if not datas:
            return False

        if not isinstance(datas, list):
            return False

        dest = installPath + '/minorPlanets.mpc'

        with open(dest, 'w') as f:
            for data in datas:
                line = ''
                line += f'{"":4s}'
                line += f'{data.get("Orbit_type", ""):1s}'
                line += f'{data.get("Provisional_packed_desig", ""):7s}'
                line += f'{"":2s}'
                line += f'{data.get("Year_of_perihelion", 0):04d}'
                line += f'{"":1s}'
                line += f'{data.get("Month_of_perihelion", 0):02d}'
                line += f'{"":1s}'
                line += f'{data.get("Day_of_perihelion", 0):7.4f}'
                line += f'{"":1s}'
                line += f'{data.get("Perihelion_dist", 0):9.6f}'
                line += f'{"":2s}'
                line += f'{data.get("e", 0):8.6f}'
                line += f'{"":2s}'
                line += f'{data.get("Peri", 0):8.4f}'
                line += f'{"":2s}'
                line += f'{data.get("Node", 0):8.4f}'
                line += f'{"":2s}'
                line += f'{data.get("i", 0):8.4f}'
                line += f'{"":2s}'

                if 'Epoch_year' in data:
                    line += f'{data.get("Epoch_year", 0):04d}'
                    line += f'{data.get("Epoch_month", 0):02d}'
                    line += f'{data.get("Epoch_day", 0):02d}'
                else:
                    line += f'{"":8s}'

                line += f'{"":2s}'
                line += f'{data.get("H", 0):4.1f}'
                line += f'{"":1s}'
                line += f'{data.get("G", 0):4.1f}'
                line += f'{"":2s}'
                line += f'{data.get("Designation_and_name", ""):56s}'
                line += f'{"":1s}'
                line += f'{data.get("Ref", " "):>9s}'
                line += '\n'
                f.write(line)

        return True

    @staticmethod
    def convertDatePacked(value):
        """
        conversion is described on
        https://www.minorplanetcenter.net/iau/info/PackedDates.html
        :param value:
        :return:
        """
        value = int(value)

        if 0 <= value < 10:
            resultChar = f'{value:1d}'

        elif 10 <= value < 36:
            resultChar = chr(ord('A') + value - 10)

        elif 36 <= value < 62:
            resultChar = chr(ord('a') + value - 36)

        else:
            resultChar = ' '

        return resultChar

    def generateCycleCountTextPacked(self, cycle):
        """
        :param cycle:
        :return:
        """
        digit1Value = cycle % 10
        digit2Value = int(cycle / 10)
        cycleChar1 = f'{digit1Value:1d}'
        cycleChar2 = self.convertDatePacked(digit2Value)
        cycleText = cycleChar2 + cycleChar1

        return cycleText

    @staticmethod
    def generateCenturyPacked(century):
        """

        :param century:
        :return:
        """
        centConvert = {'18': 'I',
                       '19': 'J',
                       '20': 'K',
                       '21': 'L'}

        centuryP = centConvert.get(century, ' ')

        return centuryP

    def generateDesignationPacked(self, designation):
        """
        description of conversion in
        https://minorplanetcenter.net//iau/info/PackedDes.html
        :param designation:
        :return:
        """
        if not designation:
            return 'xxxxxxx'

        designation = designation.strip()
        century = designation[0:2]
        centuryPacked = self.generateCenturyPacked(century)

        year = designation[2:4]
        halfMonth = designation[5]
        halfMonthOrder = designation[6]
        cycleText = designation[7:11]

        if cycleText and cycleText.isdigit():
            cycle = int(cycleText)

        else:
            cycle = 0

        cycleText = self.generateCycleCountTextPacked(cycle)

        designationPacked = f'{centuryPacked}{year}{halfMonth}{cycleText}{halfMonthOrder}'
        return designationPacked

    def generateDatePacked(self, month, day):
        """
        conversion is described on
        https://www.minorplanetcenter.net/iau/info/PackedDates.html
        :param month:
        :param day:
        :return:
        """
        dateChar1 = self.convertDatePacked(month)
        dateChar2 = self.convertDatePacked(day)

        dayPacked = dateChar1 + dateChar2
        return dayPacked

    def generateEpochPacked(self, epoch):
        """
        conversion is described on
        https://www.minorplanetcenter.net/iau/info/PackedDates.html
        :param epoch:
        :return:
        """
        if not epoch:
            return 'xxxxx'

        date = self.app.mount.obsSite.ts.tt_jd(int(epoch + 0.5))
        year, month, day = date.tt_strftime('%Y-%m-%d').split('-')
        century = year[0:2]
        centuryPacked = self.generateCenturyPacked(century)
        year = year[2:4]
        dayPacked = self.generateDatePacked(month, day)

        epochPackedText = f'{centuryPacked:1s}{year:2s}{dayPacked}'
        return epochPackedText

    def generateOldDesignationPacked(self, numberText):
        """
        :param numberText:
        :return:
        """
        if not numberText:
            return 'xxxxxxx'

        number = int(numberText.rstrip(')').lstrip('('))

        numberChar = self.convertDatePacked(number / 10000)
        designationPacked = f'{numberChar}{number % 10000:04d}  '

        return designationPacked

    def writeAsteroidMPC(self, datas=None, installPath=''):
        """
        data format of json and file description in
        https://minorplanetcenter.net/Extended_Files/Extended_MPCORB_Data_Format_Manual.pdf

        we have a mix of new and old style designation to manage. the old style seem to
        have the ley 'Number' in json, new style not.

        :param datas:
        :param installPath:
        :return:
        """
        if not datas:
            return False

        if not isinstance(datas, list):
            return False

        dest = installPath + '/minorPlanets.mpc'

        with open(dest, 'w') as f:
            for data in datas:
                line = ''

                if 'Number' in data:
                    numberText = data.get('Number', '')
                    designationPacked = self.generateOldDesignationPacked(numberText)

                else:
                    designation = data.get('Principal_desig', '')
                    designationPacked = self.generateDesignationPacked(designation)

                line += f'{designationPacked:7s}'
                line += f'{"":1s}'
                line += f'{data.get("H", 0):<5g}'
                line += f'{"":1s}'
                line += f'{data.get("G", 0):5g}'
                line += f'{"":1s}'

                epochPacked = self.generateEpochPacked(data.get('Epoch', 0))

                line += f'{epochPacked:5s}'
                line += f'{"":1s}'
                line += f'{data.get("M", 0):9.5f}'
                line += f'{"":2s}'
                line += f'{data.get("Peri", 0):9.5f}'
                line += f'{"":2s}'
                line += f'{data.get("Node", 0):9.5f}'
                line += f'{"":2s}'
                line += f'{data.get("i", 0):9.5f}'
                line += f'{"":2s}'
                line += f'{data.get("e", 0):9.7f}'
                line += f'{"":1s}'
                line += f'{data.get("n", 0):11.8f}'
                line += f'{"":1s}'
                line += f'{data.get("a", 0):11.7f}'
                line += f'{"":2s}'
                line += f'{data.get("U", ""):1s}'
                line += f'{"":1s}'
                line += f'{data.get("Ref", ""):9s}'
                line += f'{"":1s}'
                line += f'{data.get("Num_obs", 0):5.0f}'
                line += f'{"":1s}'
                line += f'{data.get("Num_opps", 0):3.0f}'

                line += f'{"":1s}'
                if 'Arc_years' in data:
                    line += f'{data.get("Arc_years",""):9s}'

                elif 'Arc_length' in data:
                    arcLength = data.get("Arc_length", 0)
                    line += f'{arcLength:4.0f} days'

                else:
                    line += f'{"":9s}'

                line += f'{"":1s}'
                line += f'{data.get("rms", 0):4.2f}'
                line += f'{"":1s}'
                line += f'{data.get("Perturbers", ""):3s}'
                line += f'{"":1s}'
                line += f'{data.get("Perturbers_2", ""):3s}'
                line += f'{"":1s}'
                line += f'{data.get("Computer", ""):10s}'
                line += f'{"":1s}'
                line += f'{data.get("Hex_flags", ""):4s}'
                line += f'{"":1s}'

                if 'Number' in data:
                    line += f'{data.get("Number", ""):>8s} {data.get("Name", ""):18s}'

                else:
                    line += f'{"":9s}'
                    line += f'{data.get("Principal_desig", ""):18s}'

                line += f'{"":1s}'
                line += f'{data.get("Last_obs", "").replace("-", ""):8s}'
                line += f'{"":1s}'
                line += f'{data.get("Tp", 0):13.5f}'
                line += '\n'
                f.write(line)

        return True

    @staticmethod
    def writeSatelliteTLE(datas=None, installPath=''):
        """
        data format of TLE and file description in

        :param datas:
        :param installPath:
        :return:
        """
        if not datas:
            return False

        if not isinstance(datas, dict):
            return False

        dest = installPath + '/satellites.tle'

        with open(dest, 'w') as f:
            for name in datas:
                line1, line2 = export_tle(datas[name].model)
                f.write(name + '\n')
                f.write(line1 + '\n')
                f.write(line2 + '\n')

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries

# external packages
import wakeonlan

# local import


class SettMount(object):
    """
    """

    def __init__(self):
        self.ui.mountOn.clicked.connect(self.mountBoot)
        self.ui.mountOff.clicked.connect(self.mountShutdown)
        self.ui.mountHost.editingFinished.connect(self.mountHost)
        self.ui.port3492.clicked.connect(self.mountHost)
        self.ui.port3490.clicked.connect(self.mountHost)
        self.ui.mountMAC.editingFinished.connect(self.mountMAC)
        self.ui.bootRackComp.clicked.connect(self.bootRackComp)
        self.app.mount.signals.settingDone.connect(self.setMountMAC)
        self.app.mount.signals.firmwareDone.connect(self.updateFwGui)
        self.ui.settleTimeMount.valueChanged.connect(self.setMountSettlingTime)

    def initConfig(self):
        """
        :return:
        """
        config = self.app.config['mainW']
        self.ui.mountHost.setText(config.get('mountHost', ''))
        self.ui.port3492.setChecked(config.get('port3492', True))
        self.mountHost()
        self.ui.mountMAC.setText(config.get('mountMAC', ''))
        self.mountMAC()
        self.ui.rackCompMAC.setText(config.get('rackCompMAC', ''))
        self.ui.settleTimeMount.setValue(config.get('settleTimeMount', 0))
        return True

    def storeConfig(self):
        """
        :return:
        """
        config = self.app.config['mainW']
        config['mountHost'] = self.ui.mountHost.text()
        config['mountMAC'] = self.ui.mountMAC.text()
        config['rackCompMAC'] = self.ui.rackCompMAC.text()
        config['settleTimeMount'] = self.ui.settleTimeMount.value()
        config['port3492'] = self.ui.port3492.isChecked()
        return True

    def mountBoot(self):
        """
        :return:
        """
        if self.app.mount.bootMount():
            self.app.message.emit('Sent boot command to mount', 0)
            return True

        else:
            self.app.message.emit('Mount cannot be booted', 2)
            return False

    def mountShutdown(self):
        """
        :return:
        """
        if self.app.mount.shutdown():
            self.app.message.emit('Shutting mount down', 0)
            return True

        else:
            self.app.message.emit('Mount cannot be shutdown', 2)
            return False

    def checkFormatMAC(self, value):
        """
        :param      value: string with mac address
        :return:    checked string in upper cases
        """
        if not value:
            self.log.info('wrong MAC value: {0}'.format(value))
            return None
        if not isinstance(value, str):
            self.log.info('wrong MAC value: {0}'.format(value))
            return None

        value = value.upper()
        value = value.replace('.', ':')
        value = value.split(':')
        if len(value) != 6:
            self.log.info('wrong MAC value: {0}'.format(value))
            return None

        for chunk in value:
            if len(chunk) != 2:
                self.log.info('wrong MAC value: {0}'.format(value))
                return None

            for char in chunk:
                if char not in ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
                                'A', 'B', 'C', 'D', 'E', 'F']:
                    self.log.info('wrong MAC value: {0}'.format(value))
                    return None

        value = '{0:2s}:{1:2s}:{2:2s}:{3:2s}:{4:2s}:{5:2s}'.format(*value)
        return value

    def bootRackComp(self):
        """
        :return:
        """
        MAC = self.ui.rackCompMAC.text()
        MAC = self.checkFormatMAC(MAC)
        if MAC is not None:
            wakeonlan.send_magic_packet(MAC)
            self.app.message.emit('Sent boot command to rack computer', 0)
            return True
        else:
            self.app.message.emit('Rack computer cannot be booted', 2)
            return False

    def mountHost(self):
        """
        :return: true for test purpose
        """
        if self.ui.port3492.isChecked():
            port = 3492
        else:
            port = 3490

        self.app.mount.host = (self.ui.mountHost.text(), port)
        return True

    def mountMAC(self):
        """
        :return: true for test purpose
        """
        self.app.mount.MAC = self.ui.mountMAC.text()
        return True

    def setMountMAC(self, sett=None):
        """
        :param sett:
        :return: true for test purpose
        """
        if sett is None:
            return False
        if sett.addressLanMAC is None:
            return False
        if not sett.addressLanMAC:
            return False

        self.app.mount.MAC = sett.addressLanMAC
        self.ui.mountMAC.setText(self.app.mount.MAC)
        return True

    def setMountSettlingTime(self):
        """
        :return: true for test purpose
        """
        self.app.mount.settlingTime = self.ui.settleTimeMount.value()
        return True

    def updateFwGui(self, fw):
        """
        :return:    True if ok for testing
        """
        self.guiSetText(self.ui.product, 's', fw.product)
        self.guiSetText(self.ui.vString, 's', fw.vString)
        self.guiSetText(self.ui.fwdate, 's', fw.date)
        self.guiSetText(self.ui.fwtime, 's', fw.time)
        self.guiSetText(self.ui.hardware, 's', fw.hardware)
        return True



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/downloadPopup.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DownloadPopup(object):
    def setupUi(self, DownloadPopup):
        DownloadPopup.setObjectName("DownloadPopup")
        DownloadPopup.resize(400, 46)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DownloadPopup.sizePolicy().hasHeightForWidth())
        DownloadPopup.setSizePolicy(sizePolicy)
        DownloadPopup.setMinimumSize(QtCore.QSize(400, 0))
        DownloadPopup.setMaximumSize(QtCore.QSize(400, 80))
        DownloadPopup.setSizeIncrement(QtCore.QSize(10, 10))
        DownloadPopup.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        DownloadPopup.setFont(font)
        self.progressBar = QtWidgets.QProgressBar(DownloadPopup)
        self.progressBar.setGeometry(QtCore.QRect(10, 10, 376, 26))
        self.progressBar.setMaximum(100)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setAlignment(QtCore.Qt.AlignCenter)
        self.progressBar.setTextVisible(True)
        self.progressBar.setOrientation(QtCore.Qt.Horizontal)
        self.progressBar.setInvertedAppearance(False)
        self.progressBar.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.progressBar.setObjectName("progressBar")

        self.retranslateUi(DownloadPopup)
        QtCore.QMetaObject.connectSlotsByName(DownloadPopup)

    def retranslateUi(self, DownloadPopup):
        _translate = QtCore.QCoreApplication.translate
        DownloadPopup.setWindowTitle(_translate("DownloadPopup", "Device Setup"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DownloadPopup = QtWidgets.QWidget()
    ui = Ui_DownloadPopup()
    ui.setupUi(DownloadPopup)
    DownloadPopup.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
#
# Licence APL2.0
#
###########################################################
# standard libraries
import glob
import json
import os

# external packages
from PyQt5.QtWidgets import QMessageBox, QLineEdit, QInputDialog
import numpy as np
import matplotlib.pyplot

# local import


class ManageModel(object):
    """
    """

    def __init__(self):
        self.runningOptimize = False
        self.fittedModelPoints = []
        self.fittedModelPath = ''
        self.plane = None

        ms = self.app.mount.signals
        ms.alignDone.connect(self.showModelPosition)
        ms.alignDone.connect(self.showErrorAscending)
        ms.alignDone.connect(self.showErrorDistribution)
        ms.namesDone.connect(self.setNameList)

        self.ui.showErrorValues.clicked.connect(self.showModelPosition)
        self.ui.showNumbers.clicked.connect(self.showModelPosition)
        self.ui.showNoAnnotation.clicked.connect(self.showModelPosition)
        self.ui.refreshName.clicked.connect(self.refreshName)
        self.ui.refreshModel.clicked.connect(self.refreshModel)
        self.ui.clearModel.clicked.connect(self.clearModel)
        self.ui.loadName.clicked.connect(self.loadName)
        self.ui.saveName.clicked.connect(self.saveName)
        self.ui.deleteName.clicked.connect(self.deleteName)
        self.ui.runOptimize.clicked.connect(self.runOptimize)
        self.ui.cancelOptimize.clicked.connect(self.cancelOptimize)
        self.ui.deleteWorstPoint.clicked.connect(self.deleteWorstPoint)
        self.ui.showActualModelAnalyse.clicked.connect(self.showActualModelAnalyse)
        self.ui.showOriginalModelAnalyse.clicked.connect(self.showOriginalModelAnalyse)

        self.ui.targetRMS.valueChanged.connect(self.showModelPosition)
        self.ui.targetRMS.valueChanged.connect(self.showErrorAscending)
        self.ui.targetRMS.valueChanged.connect(self.showErrorDistribution)

    def initConfig(self):
        """
        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.showErrorValues.setChecked(config.get('showErrorValues', False))
        self.ui.showNumbers.setChecked(config.get('showNumbers', False))
        self.ui.showNoAnnotation.setChecked(config.get('showNoAnnotation', True))
        self.ui.targetRMS.setValue(config.get('targetRMS', 10))
        self.ui.optimizeOverall.setChecked(config.get('optimizeOverall', True))
        self.ui.optimizeSingle.setChecked(config.get('optimizeSingle', True))
        self.ui.autoUpdateActualAnalyse.setChecked(config.get('autoUpdateActualAnalyse', False))
        self.showModelPosition()
        self.showErrorAscending()
        self.showErrorDistribution()
        return True

    def storeConfig(self):
        """
        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['showErrorValues'] = self.ui.showErrorValues.isChecked()
        config['showNumbers'] = self.ui.showNumbers.isChecked()
        config['showNoAnnotation'] = self.ui.showNoAnnotation.isChecked()
        config['targetRMS'] = self.ui.targetRMS.value()
        config['optimizeOverall'] = self.ui.optimizeOverall.isChecked()
        config['optimizeSingle'] = self.ui.optimizeSingle.isChecked()
        config['autoUpdateActualAnalyse'] = self.ui.autoUpdateActualAnalyse.isChecked()
        return True

    def setNameList(self, model):
        """
        setNameList populates the list of model names in the main window. before
        adding the data, the existent list will be deleted.

        :return:    True if ok for testing
        """
        self.ui.nameList.clear()
        for name in model.nameList:
            self.ui.nameList.addItem(name)
        self.ui.nameList.sortItems()
        self.ui.nameList.update()
        return True

    @staticmethod
    def findKeysFromSourceInDest(buildModel, mountModel):
        """
        :param buildModel:
        :param mountModel:
        :return: success
        """
        pointsIn = []
        pointsOut = []
        for buildPoint in buildModel:
            for mountPoint in mountModel:
                if mountModel[mountPoint] == buildModel[buildPoint]:
                    pointsIn.append(buildPoint)
                    break

            else:
                pointsOut.append(buildPoint)
        return pointsIn, pointsOut

    def compareModel(self, buildModelData, mountModel):
        """
        :param buildModelData:
        :param mountModel:
        :return:
        """
        buildModel = {}
        for star in buildModelData:
            index = star.get('errorIndex', 0)
            mount = {'ha': star.get('haMountModel', 0),
                     'dec': star.get('decMountModel', 0)}
            buildModel[index] = mount

        pointsIn, pointsOut = self.findKeysFromSourceInDest(buildModel, mountModel)
        return pointsIn, pointsOut

    def findFittingModel(self):
        """
        findFittingModel takes the actual loaded model from the mount and tries
        to find the fitting model run data. therefore it compares up to 5 points
        to find out. all optimized model files (containing opt in filename) are
        ignored.

        :return: success
        """
        mountModel = {}
        for star in self.app.mount.model.starList:
            mountModel[star.number] = {'ha': star.coord.ra.hours,
                                       'dec': star.coord.dec.degrees}

        modelFileList = glob.glob(self.app.mwGlob['modelDir'] + '/*.model')

        for modelFilePath in modelFileList:
            if 'opt' in modelFilePath:
                continue

            with open(modelFilePath, 'r') as inFile:
                try:
                    buildModelData = json.load(inFile)

                except Exception as e:
                    self.log.warning(f'Cannot load model file: {[inFile]}, error: {e}')
                    continue

            pointsIn, pointsOut = self.compareModel(buildModelData, mountModel)

            if pointsIn:
                self.fittedModelPoints = pointsIn
                self.fittedModelPath = modelFilePath
                break
        else:
            self.fittedModelPoints = []
            self.fittedModelPath = ''
            pointsIn = []
            pointsOut = []

        name = os.path.splitext(os.path.basename(self.fittedModelPath))[0]

        return name, pointsIn, pointsOut

    def showModelPosition(self):
        """
        showModelPosition draws a polar plot of the align model stars and their
        errors in color. the basic setup of the plot is taking place in the
        central widget class. which is instantiated from there. important: the
        coordinate in model is in HA and DEC  and not in RA and DEC. using
        skyfield is a little bit misleading, because you address the hour angle
         as .ra.hours

        :return:    True if ok for testing
        """
        model = self.app.mount.model
        if model is None:
            hasNoStars = True

        else:
            hasNoStars = model.starList is None or not model.starList

        axe, fig = self.generatePolar(widget=self.modelPositionPlot)

        axe.set_yticks(range(0, 90, 10))
        axe.set_ylim(0, 90)
        yLabel = ['', '', '', '', '', '', '', '', '']
        axe.set_yticklabels(yLabel)

        if hasNoStars:
            axe.figure.canvas.draw()
            return False

        altitude = np.asarray([x.alt.degrees for x in model.starList])
        azimuth = np.asarray([x.az.degrees for x in model.starList])
        error = np.asarray([x.errorRMS for x in model.starList])
        self.plane = [(alt, az) for alt, az in zip(altitude, azimuth)]

        # and plot it
        cm = matplotlib.pyplot.cm.get_cmap('RdYlGn_r')
        colors = np.asarray(error)
        scaleErrorMax = max(colors)
        scaleErrorMin = min(colors)
        area = [200 if x >= max(colors) else 60 for x in error]
        theta = azimuth / 180.0 * np.pi
        r = 90 - altitude
        scatter = axe.scatter(theta,
                              r,
                              c=colors,
                              vmin=scaleErrorMin,
                              vmax=scaleErrorMax,
                              s=area,
                              cmap=cm,
                              zorder=0,
                              )

        if self.ui.showErrorValues.isChecked():
            for star in model.starList:
                text = f'{star.errorRMS:3.1f}'
                axe.annotate(text,
                             xy=(theta[star.number - 1],
                                 r[star.number - 1]),
                             color=self.M_BLUE,
                             fontsize=9,
                             fontweight='bold',
                             zorder=1,
                             )

        elif self.ui.showNumbers.isChecked():
            for star in model.starList:
                text = f'{star.number:3.0f}'
                axe.annotate(text,
                             xy=(theta[star.number - 1],
                                 r[star.number - 1]),
                             color=self.M_BLUE,
                             fontsize=9,
                             fontweight='bold',
                             zorder=1,
                             )
        self.generateColorbar(scatter=scatter, figure=fig, label='Error [arcsec]')
        axe.figure.canvas.draw()
        return True

    def showErrorAscending(self):
        """
        showErrorAscending draws a plot of the align model stars and their
        errors in ascending order.

        :return:    True if ok for testing
        """
        model = self.app.mount.model
        if model is None:
            hasNoStars = True

        else:
            hasNoStars = model.starList is None or not model.starList

        axe, _ = self.generateFlat(widget=self.errorAscendingPlot,
                                   title='Model Point Errors in ascending order')

        if hasNoStars:
            axe.figure.canvas.draw()
            return False

        axe.set_xlabel('Star',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)
        axe.set_ylabel('Error per Star [RMS]',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)

        errors = [star.errorRMS for star in model.starList]
        errors.sort()
        index = range(0, len(errors))
        axe.plot(index,
                 errors,
                 marker='.',
                 markersize=5,
                 linestyle='none',
                 color=self.M_GREEN)

        value = self.ui.targetRMS.value()
        axe.plot([0, len(index) - 1],
                 [value, value],
                 lw=3,
                 color=self.M_PINK_H)

        axe.figure.canvas.draw()

        return True

    def showErrorDistribution(self):
        """
        showErrorDistribution draws a polar plot of the align model stars and
        their errors in color. the basic setup of the plot is taking place in the
        central widget class. which is instantiated from there. important: the
        coordinate in model is in HA and DEC  and not in RA and DEC. using
        skyfield is a little bit misleading, because you address the hour
        angle as ra.hours

        :return:    True if ok for testing
        """

        model = self.app.mount.model
        if model is None:
            hasNoStars = True

        else:
            hasNoStars = model.starList is None or not model.starList

        axe, _ = self.generatePolar(widget=self.errorDistributionPlot,
                                    title='Error Distribution')

        if hasNoStars:
            axe.figure.canvas.draw()
            return False

        angles = [star.errorAngle.degrees / 180.0 * np.pi for star in model.starList]
        errors = [star.errorRMS for star in model.starList]

        axe.plot(angles,
                 errors,
                 marker='.',
                 markersize=5,
                 linestyle='none',
                 color=self.M_GREEN)

        values = [self.ui.targetRMS.value()] * 73
        angles = np.arange(0, 365 / 180.0 * np.pi, 5 / 180.0 * np.pi)
        axe.plot(angles,
                 values,
                 lw=3,
                 color=self.M_PINK_H)

        axe.figure.canvas.draw()

        return True

    def clearRefreshName(self):
        """
        :return: True for test purpose
        """
        self.changeStyleDynamic(self.ui.refreshName, 'running', 'false')
        self.ui.deleteName.setEnabled(True)
        self.ui.saveName.setEnabled(True)
        self.ui.loadName.setEnabled(True)
        self.app.mount.signals.namesDone.disconnect(self.clearRefreshName)
        self.app.message.emit('Model names refreshed', 0)
        return True

    def refreshName(self):
        """
        refreshName disables interfering functions in gui and start reloading the
        names list for model in the mount computer. it connects a link to clearRefreshNames
        which enables the former disabled gui buttons and removes the link to the method.
        after it triggers the refresh of names, it finished, because behaviour is event
        driven

        :return: True for test purpose
        """
        self.app.mount.signals.namesDone.connect(self.clearRefreshName)
        self.ui.deleteName.setEnabled(False)
        self.ui.saveName.setEnabled(False)
        self.ui.loadName.setEnabled(False)
        self.changeStyleDynamic(self.ui.refreshName, 'running', 'true')
        self.app.mount.getNames()
        return True

    def loadName(self):
        """
        loadName take the given name and loads the stored model as the actual alignment
        model for the mount. after that it refreshes the alignment model data in
        mountwizzard

        :return: success
        """
        if self.ui.nameList.currentItem() is None:
            self.app.message.emit('No model name selected', 2)
            return False
        modelName = self.ui.nameList.currentItem().text()
        suc = self.app.mount.model.loadName(modelName)
        if not suc:
            self.app.message.emit('Model [{0}] cannot be loaded'.format(modelName), 2)
            return False
        else:
            self.app.message.emit('Model [{0}] loaded'.format(modelName), 0)
            self.refreshModel()
            return True

    def saveName(self):
        """
        saveName take the given name and saves the actual alignment model to the model
        database in the mount computer. after that it refreshes the list of the alignment
        model names in mountwizzard.

        :return: success
        """
        dlg = QInputDialog()
        modelName, ok = dlg.getText(self,
                                    'Save model',
                                    'New model name',
                                    QLineEdit.Normal,
                                    '',
                                    )
        if modelName is None or not modelName:
            self.app.message.emit('No model name given', 2)
            return False
        if not ok:
            return False

        suc = self.app.mount.model.storeName(modelName)
        if not suc:
            self.app.message.emit('Model [{0}] cannot be saved'.format(modelName), 2)
            return False
        else:
            self.app.message.emit('Model [{0}] saved'.format(modelName), 0)
            self.refreshName()
            return True

    def deleteName(self):
        """
        deleteName take the given name and deletes it from the model database in the
        mount computer. after that it refreshes the list of the alignment model names in
        mountwizzard.

        :return: success
        """
        if self.ui.nameList.currentItem() is None:
            self.app.message.emit('No model name selected', 2)
            return False

        modelName = self.ui.nameList.currentItem().text()
        msg = QMessageBox
        reply = msg.question(self,
                             'Delete model',
                             f'Delete model [{modelName}] from database?',
                             msg.Yes | msg.No,
                             msg.No,
                             )
        if reply != msg.Yes:
            return False

        suc = self.app.mount.model.deleteName(modelName)
        if not suc:
            self.app.message.emit('Model [{0}] cannot be deleted'.format(modelName), 2)
            return False
        else:
            self.app.message.emit('Model [{0}] deleted'.format(modelName), 0)
            self.refreshName()
            return True

    def writeBuildModelOptimized(self, foundModel, pointsIn, pointsOut):
        """
        :param foundModel:
        :param pointsIn:
        :param pointsOut:
        :return: true for test purpose
        """
        actPath = self.app.mwGlob['modelDir'] + '/' + foundModel + '.model'
        newPath = self.app.mwGlob['modelDir'] + '/' + foundModel + '-opt.model'

        with open(actPath) as actFile:
            actModel = json.load(actFile)

        newModel = []

        for element in actModel:
            if element['errorIndex'] in pointsOut:
                continue
            newModel.append(element)

        newModel = self.writeRetrofitData(self.app.mount.model, newModel)

        with open(newPath, 'w+') as newFile:
            json.dump(newModel, newFile, sort_keys=True, indent=4)

        return True

    def clearRefreshModel(self):
        """
        :return: True for test purpose
        """
        self.changeStyleDynamic(self.ui.refreshModel, 'running', 'false')
        self.ui.deleteWorstPoint.setEnabled(True)
        self.ui.runOptimize.setEnabled(True)
        self.ui.cancelOptimize.setEnabled(True)
        self.ui.clearModel.setEnabled(True)
        self.app.mount.signals.alignDone.disconnect(self.clearRefreshModel)
        self.app.message.emit('Align model data refreshed', 0)

        foundModel, pointsIn, pointsOut = self.findFittingModel()

        if foundModel:
            self.app.message.emit(f'Found stored model:  [{foundModel}]', 0)
            self.ui.originalModel.setText(foundModel)
            self.writeBuildModelOptimized(foundModel, pointsIn, pointsOut)

        else:
            self.ui.originalModel.setText('No fitting model file found')

        if self.ui.autoUpdateActualAnalyse.isChecked():
            self.showActualModelAnalyse()

        return True

    def refreshModel(self):
        """
        refreshModel disables interfering functions in gui and start reloading the
        alignment model from the mount computer. it connects a link to clearRefreshModel
        which enables the former disabled gui buttons and removes the link to the method.
        after it triggers the refresh of names, it finished, because behaviour is event
        driven

        :return: True for test purpose
        """
        self.changeStyleDynamic(self.ui.refreshModel, 'running', 'true')
        self.app.mount.signals.alignDone.connect(self.clearRefreshModel)
        self.ui.deleteWorstPoint.setEnabled(False)
        self.ui.runOptimize.setEnabled(False)
        self.ui.cancelOptimize.setEnabled(False)
        self.ui.clearModel.setEnabled(False)
        self.app.mount.getAlign()

        return True

    def clearModel(self):
        """
        :return:
        """
        msg = QMessageBox
        reply = msg.question(self,
                             'Clear model',
                             'Clear actual alignment model?',
                             msg.Yes | msg.No,
                             msg.No,
                             )
        if reply == msg.No:
            return False

        suc = self.app.mount.model.clearAlign()
        if not suc:
            self.app.message.emit('Actual model cannot be cleared', 2)
            return False

        else:
            self.app.message.emit('Actual model cleared', 0)
            self.refreshModel()
            return True

    def deleteWorstPoint(self):
        """
        :return:
        """
        model = self.app.mount.model
        if not model.numberStars:
            return False

        wIndex = model.starList.index(max(model.starList))
        wStar = model.starList[wIndex]
        error = wStar.errorRMS
        suc = model.deletePoint(wStar.number)
        if not suc:
            self.app.message.emit('Worst point cannot be deleted', 2)
            return False

        else:
            text = f'Point: {wIndex + 1:3.0f}, RMS of {error:5.1f}'
            text += ' arcsec deleted.'
            self.app.message.emit(text, 0)
            self.refreshModel()

        return True

    def runTargetRMS(self):
        """
        :return: True for test purpose
        """
        mount = self.app.mount
        if mount.model.errorRMS < self.ui.targetRMS.value():
            self.runningOptimize = False

        if mount.model.numberStars is None:
            numberStars = 0

        else:
            numberStars = mount.model.numberStars

        if self.runningOptimize and numberStars > 1:
            wIndex = mount.model.starList.index(max(mount.model.starList))
            wStar = mount.model.starList[wIndex]
            suc = mount.model.deletePoint(wStar.number)
            if not suc:
                self.runningOptimize = False
                self.app.message.emit(f'Star [{wStar.number + 1:3.0f}] cannot be deleted', 2)

            else:
                text = f'Point: {wStar.number + 1:3.0f}: '
                text += f'RMS of {wStar.errorRMS:5.1f} arcsec deleted.'
                self.app.message.emit(text, 0)

            mount.getAlign()

        else:
            self.finishOptimize()
        return True

    def runSingleRMS(self):
        """
        :return: True for test purpose
        """
        mount = self.app.mount
        if all([star.errorRMS < self.ui.targetRMS.value() for star in mount.model.starList]):
            self.runningOptimize = False

        if mount.model.numberStars is None:
            numberStars = 0

        else:
            numberStars = mount.model.numberStars

        if self.runningOptimize and numberStars > 1:
            wIndex = mount.model.starList.index(max(mount.model.starList))
            wStar = mount.model.starList[wIndex]
            suc = mount.model.deletePoint(wStar.number)
            if not suc:
                self.runningOptimize = False
                self.app.message.emit(f'Point {wStar.number + 1:3.0f} cannot be deleted', 2)

            else:
                text = f'Point: {wStar.number + 1:3.0f}, RMS of {wStar.errorRMS:5.1f}'
                text += ' arcsec deleted.'
                self.app.message.emit(text, 0)

            mount.getAlign()

        else:
            self.finishOptimize()
        return True

    def runOptimize(self):
        """
        :return: true for test purpose
        """
        self.app.message.emit('Start optimizing model', 2)
        self.runningOptimize = True
        self.ui.deleteWorstPoint.setEnabled(False)
        self.ui.clearModel.setEnabled(False)
        self.ui.refreshModel.setEnabled(False)
        self.changeStyleDynamic(self.ui.runOptimize, 'running', 'true')
        self.changeStyleDynamic(self.ui.cancelOptimize, 'cancel', 'true')

        if self.ui.optimizeOverall.isChecked():
            self.app.mount.signals.alignDone.connect(self.runTargetRMS)
            self.runTargetRMS()

        else:
            self.app.mount.signals.alignDone.connect(self.runSingleRMS)
            self.runSingleRMS()
        return True

    def finishOptimize(self):
        """
        :return:
        """
        if self.ui.optimizeOverall.isChecked():
            self.app.mount.signals.alignDone.disconnect(self.runTargetRMS)

        else:
            self.app.mount.signals.alignDone.disconnect(self.runSingleRMS)

        self.changeStyleDynamic(self.ui.runOptimize, 'running', 'false')
        self.ui.deleteWorstPoint.setEnabled(True)
        self.ui.clearModel.setEnabled(True)
        self.ui.refreshModel.setEnabled(True)
        self.changeStyleDynamic(self.ui.cancelOptimize, 'cancel', 'false')
        self.app.message.emit('Optimizing done', 2)

        return True

    def cancelOptimize(self):
        """
        :return: true for test purpose
        """
        self.runningOptimize = False
        return True

    def showOriginalModelAnalyse(self):
        """
        :return: True for test purpose
        """
        if not self.fittedModelPath:
            return False

        if not os.path.isfile(self.fittedModelPath):
            return False

        self.app.showAnalyse.emit(self.fittedModelPath)
        return True

    def showActualModelAnalyse(self):
        """
        :return: True for test purpose
        """
        if not self.fittedModelPath:
            return False

        temp = os.path.splitext(self.fittedModelPath)
        actualPath = temp[0] + '-opt' + temp[1]
        if not os.path.isfile(actualPath):
            return False

        self.app.showAnalyse.emit(actualPath)
        return True

    def deleteDialog(self, question):
        """
        :param question:
        :return: OK
        """
        msg = QMessageBox
        reply = msg.question(self, 'Deleting point', question, msg.Yes | msg.No, msg.No)
        if reply != msg.Yes:
            return False

        else:
            return True

    def onMouseEdit(self, event):
        """
        onMouseEdit handles the mouse event in normal mode. this means depending on the
        edit mode (horizon or model points) a left click adds a new point and right click
        deletes the selected point.

        :param event: mouse events
        :return: success
        """
        if not event.inaxes:
            return False

        if not self.plane:
            return False

        if not event.dblclick:
            return False

        event.xdata = (np.degrees(event.xdata) + 360) % 360
        event.ydata = 90 - event.ydata
        index = self.getIndexPoint(event=event, plane=self.plane, epsilon=5)
        if index is None:
            return False

        error = self.app.mount.model.starList[index].errorRMS
        text = f'Do you want to delete \npoint {index + 1:3.0f}'
        text += f'\nRMS of {error:5.1f} arcsec'
        isYes = self.deleteDialog(text)
        if not isYes:
            return False

        suc = self.app.mount.model.deletePoint(index)
        if not suc:
            self.app.message.emit(f'Point {index + 1:3.0f} cannot be deleted', 2)
            return False

        text = f'Point: {index + 1:3.0f}, RMS of {error:5.1f}'
        text += ' arcsec deleted.'
        self.app.message.emit(text, 0)
        self.refreshModel()

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging

# external packages
from skyfield.api import Angle

# local imports
from mountcontrol.connection import Connection
from mountcontrol.convert import valueToAngle
from mountcontrol.convert import valueToFloat


class TLEParams(object):
    """
    The class TLEParams inherits all information and handling of TLE tracking
    and managing attributes of the connected mount and provides the abstracted
    interface to a 10 micron mount.

        >>> tleParams = TLEParams(host='')
    """

    __all__ = ['TLEParams',
               ]

    log = logging.getLogger(__name__)

    def __init__(self):

        self._azimuth = None
        self._altitude = None
        self._ra = None
        self._dec = None
        self._jdStart = None
        self._jdEnd = None
        self._flip = None
        self._message = None
        self._l0 = None
        self._l1 = None
        self._l2 = None
        self._name = None

    @property
    def azimuth(self):
        return self._azimuth

    @azimuth.setter
    def azimuth(self, value):
        if isinstance(value, Angle):
            self._azimuth = value
            return
        self._azimuth = valueToAngle(value, preference='degrees')

    @property
    def altitude(self):
        return self._altitude

    @altitude.setter
    def altitude(self, value):
        if isinstance(value, Angle):
            self._altitude = value
            return
        self._altitude = valueToAngle(value, preference='degrees')

    @property
    def ra(self):
        return self._ra

    @ra.setter
    def ra(self, value):
        if isinstance(value, Angle):
            self._ra = value
            return
        self._ra = valueToAngle(value, preference='hours')

    @property
    def dec(self):
        return self._dec

    @dec.setter
    def dec(self, value):
        if isinstance(value, Angle):
            self._dec = value
            return
        self._dec = valueToAngle(value, preference='degrees')

    @property
    def flip(self):
        return self._flip

    @flip.setter
    def flip(self, value):
        if isinstance(value, bool):
            self._flip = value
            return
        self._flip = bool(value == 'F')

    @property
    def jdStart(self):
        return self._jdStart

    @jdStart.setter
    def jdStart(self, value):
        value = valueToFloat(value)
        if value:
            self._jdStart = value
        else:
            self._jdStart = None

    @property
    def jdEnd(self):
        return self._jdEnd

    @jdEnd.setter
    def jdEnd(self, value):
        value = valueToFloat(value)
        if value:
            self._jdEnd = value
        else:
            self._jdEnd = None

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        if value:
            self._message = value
        else:
            self._message = None

    @property
    def l0(self):
        return self._l0

    @l0.setter
    def l0(self, value):
        self._l0 = value

    @property
    def l1(self):
        return self._l1

    @l1.setter
    def l1(self, value):
        self._l1 = value

    @property
    def l2(self):
        return self._l2

    @l2.setter
    def l2(self, value):
        self._l2 = value

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value


class Satellite(object):
    """
    The class Satellite inherits all information and handling of TLE tracking
    and managing attributes of the connected mount and provides the abstracted
    interface to a 10 micron mount.
    Some of the basic commands of the 10micron mount are combined and sent as
    a bulk message. idea of the procedure is to program the TLE parameters
    first, than calculate the parameters with the mount computer. this could
    be done in a cyclic way, so that you could update theses results over time.
    As the calculation takes some time this call will also block your
    application some time. finally you could start the slewing process
    where the mount slews normally to the start of the satellite track and
    waits for it. when satellite reaches the altitude minimum, the mount
    starts to track.

        >>> fw = Satellite(host='')
    """

    __all__ = ['Satellite',
               ]

    log = logging.getLogger(__name__)

    TLES = {
        'E': 'No transit pre calculated',
        'F': 'Slew failed',
        'V': 'Slewing to start and track',
        'S': 'Slewing to catch satellite',
        'Q': 'Transit ended, no tracking',
    }

    TLESCK = {
        'V': 'Slewing to the start of the transit',
        'P': 'Waiting for the satellite',
        'S': 'Slewing to catch satellite',
        'T': 'Tracking the satellite',
        'Q': 'Transit ended, no tracking',
        'E': 'No slew to satellite requested'
    }

    def __init__(self,
                 host=None,
                 ):

        self.host = host
        self.tleParams = TLEParams()

    def parseGetTLE(self, response, numberOfChunks):
        """
        Parsing the GetTLE command and entering the resulting parameters. if no
        TLE parameters could be retrieved, they were set to None

        :param response:        data load from mount
        :param numberOfChunks:  amount of parts
        :return: success:       True if ok, False if not
        """

        if len(response) != numberOfChunks:
            self.log.warning('wrong number of chunks')
            return False

        if response[0] == 'E':
            return False

        if numberOfChunks != 1:
            return False

        lines = response[0].split('$0A')

        if len(lines) != 4:
            return False

        self.tleParams.name = lines[0].rstrip()
        self.tleParams.l0 = lines[0]
        self.tleParams.l1 = lines[1]
        self.tleParams.l2 = lines[2]

        return True

    def getTLE(self):
        """
        Returns the currently-loaded two line elements.

        Returns:
        <two line elements>#    a string containing the two line elements.
                                Lines a terminated by the ASCII newline
                                character (ASCII code 10),
                                and the entire string is escaped with the
                                mechanism described in the "escaped strings"
                                section below. If no TLE is currently loaded,
                                returns the string E#.

        :return: success
        """

        conn = Connection(self.host)
        suc, response, numberOfChunks = conn.communicate(':TLEG#')

        if not suc:
            return False

        suc = self.parseGetTLE(response, numberOfChunks)
        return suc

    def setTLE(self, line0='', line1='', line2=''):
        """
        Loads satellite orbital elements in two-line format directly from the
        command protocol. <two line element> is a string containing the two
        line elements. Each lines can be terminated by escaped newline (ASCII
        code 10), carriage return (ASCII code 13) or a combination of both. The
        first line may contain the satellite name. The entire string is escaped
        with the mechanism described in the "escaped strings" section below.

        The TLE format is described here:
        https://www.celestrak.com/NORAD/documentation/tle-fmt.asp
        For example, loading the NOAA 14 element set of that page can be
        accomplished with:

        TLEL0NOAA·14·················
        1·23455U·94089A···97320.90946019··.00000140··00000-0··10191-3·0··2621
        2·23455··99.0090·272.6745·0008546·223.1686·136.8816·14.11711747148495

        Returns:
        E# invalid format
        V# valid format

        :param line0:
        :param line1:
        :param line2:
        :return: success
        """

        if not line0 or not line1 or not line2:
            return False
        if len(line1) != 69:
            return False
        if len(line2) != 69:
            return False

        commandString = f':TLEL0{line0}$0a{line1}$0a{line2}#'
        conn = Connection(self.host)
        suc, response, numberOfChunks = conn.communicate(commandString)

        if not suc:
            return False
        if response == 'E':
            return False
        if numberOfChunks != 1:
            return False

        return suc

    def parseCalcTLE(self, response, numberOfChunks):
        """
        Parsing the CalcTLE command and entering the resulting parameters. if
        no calculation could be made, the parameters were set to None

        :param response:        data load from mount
        :param numberOfChunks:  amount of parts
        :return: success:       True if ok, False if not
        """

        if len(response) != numberOfChunks:
            self.log.warning('wrong number of chunks')
            return False

        if len(response) != 3:
            self.log.warning('wrong number of chunks')
            return False

        if response[0] == 'E':
            return False
        if response[1] == 'E':
            return False
        if response[2] == 'E':
            return False

        value = response[0].split(',')
        if len(value) != 2:
            return False
        alt, az = value

        value = response[1].split(',')
        if len(value) != 2:
            return False
        ra, dec = value

        value = response[2].split(',')
        if len(value) == 3:
            start, end, flip = value
        elif len(value) == 1:
            flip = value
            start = None
            end = None
        else:
            return False

        self.tleParams.altitude = alt
        self.tleParams.azimuth = az
        self.tleParams.ra = ra
        self.tleParams.dec = dec
        self.tleParams.flip = flip
        self.tleParams.jdStart = start
        self.tleParams.jdEnd = end

        return True

    def calcTLE(self, julD='', duration=1440):
        """
        set of three commands !

        Pre calculates the first transit of the satellite with the currently
        loaded orbital elements, starting from Julian Date JD and for a period
        of min minutes, where min is from 1 to 1440. Two-line elements have to
        be loaded with the :TLEL command.

        Returns:
        E#                  no TLE loaded
        +AA.AAAA,ZZZ.ZZZZ#  the apparent alt azimuth coordinates,
                            where +AA.AAAA is the altitude in degrees with
                            decimals accounting for refraction,
                            and ZZZ.ZZZZ is the azimuth in degrees with
                            decimals.

        Returns:
        E# no TLE loaded
        RR.RRRRR,+DD.DDDD#  the apparent equatorial coordinates,
                            where RR.RRRRR is the
                            right ascension in hours with decimals,
                            and +DD.DDDD is the
                            declination in degrees with decimals.

        Gets the apparent alt azimuth coordinates of the satellite with the
        currently loaded orbital elements, computed for Julian Date JD (UTC).

        Returns:
        E#                      no TLE loaded or invalid command
        N#                      no passes in the given amount of time
        JD start,JD end,flags#  data for the first pass in the given interval.
                                JD start and JD end mark the beginning and the
                                end of the given transit. Flags is a string
                                which can be empty or contain the letter F –
                                meaning that mount will flip during the transit.

        Gets the apparent equatorial coordinates of the satellite with the
        currently loaded orbital elements, computed for Julian Date JD (UTC).

        :param julD:  julianDate as string
        :param duration:    duration in minutes
        :return: success
        """
        if not julD:
            return False
        if not 0 < duration < 1441:
            return False

        conn = Connection(self.host)
        command = f':TLEGAZ{julD}#:TLEGEQ{julD}#:TLEP{julD},{duration}#'
        suc, response, numberOfChunks = conn.communicate(command)

        if not suc:
            return False

        suc = self.parseCalcTLE(response, numberOfChunks)
        return suc

    def slewTLE(self):
        """
        Slews to the start of the satellite transit that has been pre
        calculated with the :TLEP command.

        Returns:
        E# no transit has been pre calculated
        F# slew failed due to mount parked or other status blocking slews
        V# slewing to the start of the transit, the mount will automatically
            starts tracking the satellite.
        S# the transit has already started, slewing to catch the satellite
        Q# the transit has already ended, no slew occurs

        :return: success
        """

        conn = Connection(self.host)
        suc, response, numberOfChunks = conn.communicate(':TLES#')

        if numberOfChunks != 1:
            return False, 'Error'

        message = self.TLES.get(response[0], 'Error')

        return suc, message

    def parseStatTLE(self, response, numberOfChunks):
        """
        Parsing the statTLE slow command.

        :param response:        data load from mount
        :param numberOfChunks:  amount of parts
        :return: success:       True if ok, False if not
        """

        if len(response) != numberOfChunks:
            self.log.warning('wrong number of chunks')
            return False

        if len(response) != 1:
            self.log.warning('wrong number of chunks')
            return False

        if not response[0]:
            return False

        self.tleParams.message = self.TLESCK.get(response[0], 'Error')

        return True

    def statTLE(self):
        """
        Gets the status of the slew to a pre calculated satellite transit.

        Returns:
        V# slewing to the start of the transit
        P# stopped at the start of the transit, waiting for the satellite
        S# slewing to catch the satellite
        T# tracking the satellite
        Q# the transit has ended, not tracking
        E# no slew to a satellite has been requested.

        :return: success
        """

        conn = Connection(self.host)
        suc, response, numberOfChunks = conn.communicate(':TLESCK#')

        if not suc:
            return False

        suc = self.parseStatTLE(response, numberOfChunks)
        return suc



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './mw4/gui/widgets/devicePopup.ui'
#
# Created by: PyQt5 UI code generator 5.15.2
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DevicePopup(object):
    def setupUi(self, DevicePopup):
        DevicePopup.setObjectName("DevicePopup")
        DevicePopup.resize(460, 300)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(DevicePopup.sizePolicy().hasHeightForWidth())
        DevicePopup.setSizePolicy(sizePolicy)
        DevicePopup.setMinimumSize(QtCore.QSize(460, 300))
        DevicePopup.setMaximumSize(QtCore.QSize(460, 300))
        DevicePopup.setSizeIncrement(QtCore.QSize(10, 10))
        DevicePopup.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        DevicePopup.setFont(font)
        self.tab = QtWidgets.QTabWidget(DevicePopup)
        self.tab.setGeometry(QtCore.QRect(80, 5, 371, 256))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.tab.setFont(font)
        self.tab.setObjectName("tab")
        self.ALPACA = QtWidgets.QWidget()
        self.ALPACA.setObjectName("ALPACA")
        self.label_mountIP_7 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_7.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_7.setFont(font)
        self.label_mountIP_7.setObjectName("label_mountIP_7")
        self.alpacaPort = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaPort.setGeometry(QtCore.QRect(100, 70, 50, 21))
        self.alpacaPort.setMinimumSize(QtCore.QSize(0, 0))
        self.alpacaPort.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaPort.setFont(font)
        self.alpacaPort.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaPort.setObjectName("alpacaPort")
        self.label_mountIP_8 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_8.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_8.setFont(font)
        self.label_mountIP_8.setObjectName("label_mountIP_8")
        self.label_mountIP_10 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_10.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_10.setFont(font)
        self.label_mountIP_10.setObjectName("label_mountIP_10")
        self.alpacaHostAddress = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaHostAddress.setGeometry(QtCore.QRect(100, 40, 201, 21))
        self.alpacaHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaHostAddress.setFont(font)
        self.alpacaHostAddress.setMouseTracking(False)
        self.alpacaHostAddress.setAcceptDrops(False)
        self.alpacaHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaHostAddress.setMaxLength(50)
        self.alpacaHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaHostAddress.setDragEnabled(True)
        self.alpacaHostAddress.setReadOnly(False)
        self.alpacaHostAddress.setObjectName("alpacaHostAddress")
        self.label_mountIP_11 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_11.setGeometry(QtCore.QRect(10, 130, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_11.setFont(font)
        self.label_mountIP_11.setObjectName("label_mountIP_11")
        self.label_mountIP_12 = QtWidgets.QLabel(self.ALPACA)
        self.label_mountIP_12.setGeometry(QtCore.QRect(10, 160, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_12.setFont(font)
        self.label_mountIP_12.setObjectName("label_mountIP_12")
        self.alpacaUser = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaUser.setGeometry(QtCore.QRect(100, 130, 201, 21))
        self.alpacaUser.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaUser.setFont(font)
        self.alpacaUser.setMouseTracking(False)
        self.alpacaUser.setAcceptDrops(False)
        self.alpacaUser.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaUser.setText("")
        self.alpacaUser.setMaxLength(50)
        self.alpacaUser.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaUser.setDragEnabled(True)
        self.alpacaUser.setReadOnly(False)
        self.alpacaUser.setObjectName("alpacaUser")
        self.alpacaPassword = QtWidgets.QLineEdit(self.ALPACA)
        self.alpacaPassword.setGeometry(QtCore.QRect(100, 160, 201, 21))
        self.alpacaPassword.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.alpacaPassword.setFont(font)
        self.alpacaPassword.setMouseTracking(False)
        self.alpacaPassword.setAcceptDrops(False)
        self.alpacaPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.alpacaPassword.setText("")
        self.alpacaPassword.setMaxLength(50)
        self.alpacaPassword.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.alpacaPassword.setDragEnabled(True)
        self.alpacaPassword.setReadOnly(False)
        self.alpacaPassword.setObjectName("alpacaPassword")
        self.alpacaDeviceList = QtWidgets.QComboBox(self.ALPACA)
        self.alpacaDeviceList.setGeometry(QtCore.QRect(100, 100, 201, 21))
        self.alpacaDeviceList.setObjectName("alpacaDeviceList")
        self.alpacaCopyConfig = QtWidgets.QCheckBox(self.ALPACA)
        self.alpacaCopyConfig.setEnabled(True)
        self.alpacaCopyConfig.setGeometry(QtCore.QRect(100, 190, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaCopyConfig.setFont(font)
        self.alpacaCopyConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.alpacaCopyConfig.setChecked(False)
        self.alpacaCopyConfig.setTristate(False)
        self.alpacaCopyConfig.setObjectName("alpacaCopyConfig")
        self.alpacaDiscover = QtWidgets.QPushButton(self.ALPACA)
        self.alpacaDiscover.setGeometry(QtCore.QRect(305, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.alpacaDiscover.setFont(font)
        self.alpacaDiscover.setObjectName("alpacaDiscover")
        self.tab.addTab(self.ALPACA, "")
        self.ASCOM = QtWidgets.QWidget()
        self.ASCOM.setObjectName("ASCOM")
        self.ascomSelector = QtWidgets.QPushButton(self.ASCOM)
        self.ascomSelector.setGeometry(QtCore.QRect(100, 190, 201, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.ascomSelector.setFont(font)
        self.ascomSelector.setObjectName("ascomSelector")
        self.label_mountIP_15 = QtWidgets.QLabel(self.ASCOM)
        self.label_mountIP_15.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_15.setFont(font)
        self.label_mountIP_15.setObjectName("label_mountIP_15")
        self.ascomDevice = QtWidgets.QLineEdit(self.ASCOM)
        self.ascomDevice.setGeometry(QtCore.QRect(100, 100, 196, 21))
        self.ascomDevice.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.ascomDevice.setFont(font)
        self.ascomDevice.setMouseTracking(False)
        self.ascomDevice.setAcceptDrops(False)
        self.ascomDevice.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.ascomDevice.setText("")
        self.ascomDevice.setMaxLength(50)
        self.ascomDevice.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.ascomDevice.setDragEnabled(True)
        self.ascomDevice.setReadOnly(False)
        self.ascomDevice.setObjectName("ascomDevice")
        self.tab.addTab(self.ASCOM, "")
        self.INDI = QtWidgets.QWidget()
        self.INDI.setObjectName("INDI")
        self.label_mountIP_3 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_3.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_3.setFont(font)
        self.label_mountIP_3.setObjectName("label_mountIP_3")
        self.indiHostAddress = QtWidgets.QLineEdit(self.INDI)
        self.indiHostAddress.setGeometry(QtCore.QRect(100, 40, 201, 21))
        self.indiHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.indiHostAddress.setFont(font)
        self.indiHostAddress.setMouseTracking(False)
        self.indiHostAddress.setAcceptDrops(False)
        self.indiHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.indiHostAddress.setMaxLength(50)
        self.indiHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.indiHostAddress.setDragEnabled(True)
        self.indiHostAddress.setReadOnly(False)
        self.indiHostAddress.setObjectName("indiHostAddress")
        self.label_mountIP_4 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_4.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_4.setFont(font)
        self.label_mountIP_4.setObjectName("label_mountIP_4")
        self.indiDiscover = QtWidgets.QPushButton(self.INDI)
        self.indiDiscover.setGeometry(QtCore.QRect(305, 100, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiDiscover.setFont(font)
        self.indiDiscover.setObjectName("indiDiscover")
        self.indiDeviceList = QtWidgets.QComboBox(self.INDI)
        self.indiDeviceList.setGeometry(QtCore.QRect(100, 100, 201, 21))
        self.indiDeviceList.setObjectName("indiDeviceList")
        self.label_mountIP_5 = QtWidgets.QLabel(self.INDI)
        self.label_mountIP_5.setGeometry(QtCore.QRect(10, 100, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_5.setFont(font)
        self.label_mountIP_5.setObjectName("label_mountIP_5")
        self.indiMessages = QtWidgets.QCheckBox(self.INDI)
        self.indiMessages.setGeometry(QtCore.QRect(100, 135, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiMessages.setFont(font)
        self.indiMessages.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiMessages.setChecked(False)
        self.indiMessages.setObjectName("indiMessages")
        self.indiPort = QtWidgets.QLineEdit(self.INDI)
        self.indiPort.setGeometry(QtCore.QRect(100, 70, 50, 21))
        self.indiPort.setMinimumSize(QtCore.QSize(0, 0))
        self.indiPort.setMaximumSize(QtCore.QSize(50, 16777215))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiPort.setFont(font)
        self.indiPort.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.indiPort.setObjectName("indiPort")
        self.indiLoadConfig = QtWidgets.QCheckBox(self.INDI)
        self.indiLoadConfig.setEnabled(True)
        self.indiLoadConfig.setGeometry(QtCore.QRect(100, 160, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiLoadConfig.setFont(font)
        self.indiLoadConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiLoadConfig.setChecked(False)
        self.indiLoadConfig.setTristate(False)
        self.indiLoadConfig.setObjectName("indiLoadConfig")
        self.indiCopyConfig = QtWidgets.QCheckBox(self.INDI)
        self.indiCopyConfig.setEnabled(True)
        self.indiCopyConfig.setGeometry(QtCore.QRect(100, 185, 201, 20))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.indiCopyConfig.setFont(font)
        self.indiCopyConfig.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.indiCopyConfig.setChecked(False)
        self.indiCopyConfig.setTristate(False)
        self.indiCopyConfig.setObjectName("indiCopyConfig")
        self.tab.addTab(self.INDI, "")
        self.ASTROMETRY = QtWidgets.QWidget()
        self.ASTROMETRY.setObjectName("ASTROMETRY")
        self.label_mountIP_14 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_14.setGeometry(QtCore.QRect(10, 65, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_14.setFont(font)
        self.label_mountIP_14.setObjectName("label_mountIP_14")
        self.astrometryIndexPath = QtWidgets.QLineEdit(self.ASTROMETRY)
        self.astrometryIndexPath.setEnabled(True)
        self.astrometryIndexPath.setGeometry(QtCore.QRect(10, 90, 346, 21))
        self.astrometryIndexPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryIndexPath.setFont(font)
        self.astrometryIndexPath.setMouseTracking(False)
        self.astrometryIndexPath.setAcceptDrops(False)
        self.astrometryIndexPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astrometryIndexPath.setMaxLength(100)
        self.astrometryIndexPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryIndexPath.setDragEnabled(True)
        self.astrometryIndexPath.setReadOnly(False)
        self.astrometryIndexPath.setObjectName("astrometryIndexPath")
        self.selectAstrometryIndexPath = QtWidgets.QPushButton(self.ASTROMETRY)
        self.selectAstrometryIndexPath.setGeometry(QtCore.QRect(310, 65, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstrometryIndexPath.setFont(font)
        self.selectAstrometryIndexPath.setObjectName("selectAstrometryIndexPath")
        self.astrometryDeviceList = QtWidgets.QComboBox(self.ASTROMETRY)
        self.astrometryDeviceList.setGeometry(QtCore.QRect(100, 120, 201, 21))
        self.astrometryDeviceList.setObjectName("astrometryDeviceList")
        self.label_mountIP_6 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_6.setGeometry(QtCore.QRect(10, 120, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_6.setFont(font)
        self.label_mountIP_6.setObjectName("label_mountIP_6")
        self.label_316 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_316.setGeometry(QtCore.QRect(10, 180, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_316.setFont(font)
        self.label_316.setObjectName("label_316")
        self.astrometrySearchRadius = QtWidgets.QDoubleSpinBox(self.ASTROMETRY)
        self.astrometrySearchRadius.setGeometry(QtCore.QRect(100, 150, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometrySearchRadius.setFont(font)
        self.astrometrySearchRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometrySearchRadius.setDecimals(0)
        self.astrometrySearchRadius.setMinimum(1.0)
        self.astrometrySearchRadius.setMaximum(180.0)
        self.astrometrySearchRadius.setSingleStep(5.0)
        self.astrometrySearchRadius.setProperty("value", 25.0)
        self.astrometrySearchRadius.setObjectName("astrometrySearchRadius")
        self.astrometryTimeout = QtWidgets.QDoubleSpinBox(self.ASTROMETRY)
        self.astrometryTimeout.setGeometry(QtCore.QRect(100, 180, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryTimeout.setFont(font)
        self.astrometryTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astrometryTimeout.setDecimals(0)
        self.astrometryTimeout.setMinimum(2.0)
        self.astrometryTimeout.setMaximum(300.0)
        self.astrometryTimeout.setSingleStep(5.0)
        self.astrometryTimeout.setProperty("value", 30.0)
        self.astrometryTimeout.setObjectName("astrometryTimeout")
        self.label_315 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_315.setGeometry(QtCore.QRect(10, 150, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_315.setFont(font)
        self.label_315.setObjectName("label_315")
        self.label_317 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_317.setGeometry(QtCore.QRect(160, 180, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_317.setFont(font)
        self.label_317.setObjectName("label_317")
        self.label_318 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_318.setGeometry(QtCore.QRect(160, 150, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_318.setFont(font)
        self.label_318.setObjectName("label_318")
        self.label_mountIP_17 = QtWidgets.QLabel(self.ASTROMETRY)
        self.label_mountIP_17.setGeometry(QtCore.QRect(10, 10, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_17.setFont(font)
        self.label_mountIP_17.setObjectName("label_mountIP_17")
        self.astrometryAppPath = QtWidgets.QLineEdit(self.ASTROMETRY)
        self.astrometryAppPath.setEnabled(True)
        self.astrometryAppPath.setGeometry(QtCore.QRect(10, 35, 346, 21))
        self.astrometryAppPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astrometryAppPath.setFont(font)
        self.astrometryAppPath.setMouseTracking(False)
        self.astrometryAppPath.setAcceptDrops(False)
        self.astrometryAppPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astrometryAppPath.setMaxLength(100)
        self.astrometryAppPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astrometryAppPath.setDragEnabled(True)
        self.astrometryAppPath.setReadOnly(False)
        self.astrometryAppPath.setObjectName("astrometryAppPath")
        self.selectAstrometryAppPath = QtWidgets.QPushButton(self.ASTROMETRY)
        self.selectAstrometryAppPath.setEnabled(True)
        self.selectAstrometryAppPath.setGeometry(QtCore.QRect(310, 10, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstrometryAppPath.setFont(font)
        self.selectAstrometryAppPath.setObjectName("selectAstrometryAppPath")
        self.tab.addTab(self.ASTROMETRY, "")
        self.ASTAP = QtWidgets.QWidget()
        self.ASTAP.setObjectName("ASTAP")
        self.label_mountIP_16 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_16.setGeometry(QtCore.QRect(10, 120, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_16.setFont(font)
        self.label_mountIP_16.setObjectName("label_mountIP_16")
        self.astapTimeout = QtWidgets.QDoubleSpinBox(self.ASTAP)
        self.astapTimeout.setGeometry(QtCore.QRect(100, 180, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapTimeout.setFont(font)
        self.astapTimeout.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astapTimeout.setDecimals(0)
        self.astapTimeout.setMinimum(2.0)
        self.astapTimeout.setMaximum(300.0)
        self.astapTimeout.setSingleStep(5.0)
        self.astapTimeout.setProperty("value", 30.0)
        self.astapTimeout.setObjectName("astapTimeout")
        self.astapSearchRadius = QtWidgets.QDoubleSpinBox(self.ASTAP)
        self.astapSearchRadius.setGeometry(QtCore.QRect(100, 150, 51, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapSearchRadius.setFont(font)
        self.astapSearchRadius.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.astapSearchRadius.setDecimals(0)
        self.astapSearchRadius.setMinimum(1.0)
        self.astapSearchRadius.setMaximum(180.0)
        self.astapSearchRadius.setSingleStep(5.0)
        self.astapSearchRadius.setProperty("value", 25.0)
        self.astapSearchRadius.setObjectName("astapSearchRadius")
        self.label_319 = QtWidgets.QLabel(self.ASTAP)
        self.label_319.setGeometry(QtCore.QRect(10, 180, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_319.setFont(font)
        self.label_319.setObjectName("label_319")
        self.label_320 = QtWidgets.QLabel(self.ASTAP)
        self.label_320.setGeometry(QtCore.QRect(10, 150, 86, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_320.setFont(font)
        self.label_320.setObjectName("label_320")
        self.label_321 = QtWidgets.QLabel(self.ASTAP)
        self.label_321.setGeometry(QtCore.QRect(160, 180, 21, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_321.setFont(font)
        self.label_321.setObjectName("label_321")
        self.label_322 = QtWidgets.QLabel(self.ASTAP)
        self.label_322.setGeometry(QtCore.QRect(160, 150, 61, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.label_322.setFont(font)
        self.label_322.setObjectName("label_322")
        self.astapIndexPath = QtWidgets.QLineEdit(self.ASTAP)
        self.astapIndexPath.setEnabled(True)
        self.astapIndexPath.setGeometry(QtCore.QRect(10, 90, 346, 21))
        self.astapIndexPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapIndexPath.setFont(font)
        self.astapIndexPath.setMouseTracking(False)
        self.astapIndexPath.setAcceptDrops(False)
        self.astapIndexPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astapIndexPath.setMaxLength(100)
        self.astapIndexPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astapIndexPath.setDragEnabled(True)
        self.astapIndexPath.setReadOnly(False)
        self.astapIndexPath.setObjectName("astapIndexPath")
        self.astapAppPath = QtWidgets.QLineEdit(self.ASTAP)
        self.astapAppPath.setEnabled(True)
        self.astapAppPath.setGeometry(QtCore.QRect(10, 35, 346, 21))
        self.astapAppPath.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.astapAppPath.setFont(font)
        self.astapAppPath.setMouseTracking(False)
        self.astapAppPath.setAcceptDrops(False)
        self.astapAppPath.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.astapAppPath.setMaxLength(100)
        self.astapAppPath.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.astapAppPath.setDragEnabled(True)
        self.astapAppPath.setReadOnly(False)
        self.astapAppPath.setObjectName("astapAppPath")
        self.selectAstapIndexPath = QtWidgets.QPushButton(self.ASTAP)
        self.selectAstapIndexPath.setEnabled(True)
        self.selectAstapIndexPath.setGeometry(QtCore.QRect(310, 65, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstapIndexPath.setFont(font)
        self.selectAstapIndexPath.setObjectName("selectAstapIndexPath")
        self.selectAstapAppPath = QtWidgets.QPushButton(self.ASTAP)
        self.selectAstapAppPath.setEnabled(True)
        self.selectAstapAppPath.setGeometry(QtCore.QRect(310, 10, 46, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.selectAstapAppPath.setFont(font)
        self.selectAstapAppPath.setObjectName("selectAstapAppPath")
        self.astapDeviceList = QtWidgets.QComboBox(self.ASTAP)
        self.astapDeviceList.setGeometry(QtCore.QRect(100, 120, 201, 21))
        self.astapDeviceList.setObjectName("astapDeviceList")
        self.label_mountIP_18 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_18.setGeometry(QtCore.QRect(10, 65, 146, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_18.setFont(font)
        self.label_mountIP_18.setObjectName("label_mountIP_18")
        self.label_mountIP_19 = QtWidgets.QLabel(self.ASTAP)
        self.label_mountIP_19.setGeometry(QtCore.QRect(10, 10, 106, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_19.setFont(font)
        self.label_mountIP_19.setObjectName("label_mountIP_19")
        self.tab.addTab(self.ASTAP, "")
        self.weather = QtWidgets.QWidget()
        self.weather.setObjectName("weather")
        self.label_mountIP_33 = QtWidgets.QLabel(self.weather)
        self.label_mountIP_33.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_33.setFont(font)
        self.label_mountIP_33.setObjectName("label_mountIP_33")
        self.onlineWeatherHostAddress = QtWidgets.QLineEdit(self.weather)
        self.onlineWeatherHostAddress.setEnabled(True)
        self.onlineWeatherHostAddress.setGeometry(QtCore.QRect(100, 40, 241, 21))
        self.onlineWeatherHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherHostAddress.setFont(font)
        self.onlineWeatherHostAddress.setMouseTracking(False)
        self.onlineWeatherHostAddress.setAcceptDrops(False)
        self.onlineWeatherHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherHostAddress.setMaxLength(50)
        self.onlineWeatherHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.onlineWeatherHostAddress.setDragEnabled(True)
        self.onlineWeatherHostAddress.setReadOnly(False)
        self.onlineWeatherHostAddress.setObjectName("onlineWeatherHostAddress")
        self.onlineWeatherApiKey = QtWidgets.QLineEdit(self.weather)
        self.onlineWeatherApiKey.setGeometry(QtCore.QRect(100, 70, 241, 21))
        self.onlineWeatherApiKey.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.onlineWeatherApiKey.setFont(font)
        self.onlineWeatherApiKey.setMouseTracking(False)
        self.onlineWeatherApiKey.setAcceptDrops(False)
        self.onlineWeatherApiKey.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.onlineWeatherApiKey.setText("")
        self.onlineWeatherApiKey.setMaxLength(50)
        self.onlineWeatherApiKey.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.onlineWeatherApiKey.setDragEnabled(True)
        self.onlineWeatherApiKey.setReadOnly(False)
        self.onlineWeatherApiKey.setObjectName("onlineWeatherApiKey")
        self.label_mountIP_34 = QtWidgets.QLabel(self.weather)
        self.label_mountIP_34.setGeometry(QtCore.QRect(10, 70, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_34.setFont(font)
        self.label_mountIP_34.setObjectName("label_mountIP_34")
        self.tab.addTab(self.weather, "")
        self.relay = QtWidgets.QWidget()
        self.relay.setObjectName("relay")
        self.label_mountIP_35 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_35.setGeometry(QtCore.QRect(10, 40, 81, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_35.setFont(font)
        self.label_mountIP_35.setObjectName("label_mountIP_35")
        self.relayHostAddress = QtWidgets.QLineEdit(self.relay)
        self.relayHostAddress.setEnabled(True)
        self.relayHostAddress.setGeometry(QtCore.QRect(100, 40, 241, 21))
        self.relayHostAddress.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayHostAddress.setFont(font)
        self.relayHostAddress.setMouseTracking(False)
        self.relayHostAddress.setAcceptDrops(False)
        self.relayHostAddress.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayHostAddress.setText("")
        self.relayHostAddress.setMaxLength(50)
        self.relayHostAddress.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayHostAddress.setDragEnabled(True)
        self.relayHostAddress.setReadOnly(False)
        self.relayHostAddress.setObjectName("relayHostAddress")
        self.relayPassword = QtWidgets.QLineEdit(self.relay)
        self.relayPassword.setGeometry(QtCore.QRect(100, 100, 241, 21))
        self.relayPassword.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayPassword.setFont(font)
        self.relayPassword.setMouseTracking(False)
        self.relayPassword.setAcceptDrops(False)
        self.relayPassword.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayPassword.setText("")
        self.relayPassword.setMaxLength(50)
        self.relayPassword.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayPassword.setDragEnabled(True)
        self.relayPassword.setReadOnly(False)
        self.relayPassword.setObjectName("relayPassword")
        self.relayUser = QtWidgets.QLineEdit(self.relay)
        self.relayUser.setGeometry(QtCore.QRect(100, 70, 241, 21))
        self.relayUser.setMinimumSize(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        font.setBold(False)
        font.setWeight(50)
        self.relayUser.setFont(font)
        self.relayUser.setMouseTracking(False)
        self.relayUser.setAcceptDrops(False)
        self.relayUser.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.relayUser.setText("")
        self.relayUser.setMaxLength(50)
        self.relayUser.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.relayUser.setDragEnabled(True)
        self.relayUser.setReadOnly(False)
        self.relayUser.setObjectName("relayUser")
        self.label_mountIP_36 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_36.setGeometry(QtCore.QRect(10, 70, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_36.setFont(font)
        self.label_mountIP_36.setObjectName("label_mountIP_36")
        self.label_mountIP_37 = QtWidgets.QLabel(self.relay)
        self.label_mountIP_37.setGeometry(QtCore.QRect(10, 100, 91, 21))
        font = QtGui.QFont()
        font.setFamily("Arial")
        font.setPointSize(10)
        self.label_mountIP_37.setFont(font)
        self.label_mountIP_37.setObjectName("label_mountIP_37")
        self.tab.addTab(self.relay, "")
        self.ok = QtWidgets.QPushButton(DevicePopup)
        self.ok.setGeometry(QtCore.QRect(370, 270, 75, 21))
        self.ok.setObjectName("ok")
        self.cancel = QtWidgets.QPushButton(DevicePopup)
        self.cancel.setGeometry(QtCore.QRect(270, 270, 75, 21))
        self.cancel.setObjectName("cancel")
        self.iconPixmap = QtWidgets.QLabel(DevicePopup)
        self.iconPixmap.setGeometry(QtCore.QRect(10, 110, 64, 64))
        self.iconPixmap.setObjectName("iconPixmap")

        self.retranslateUi(DevicePopup)
        self.tab.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(DevicePopup)
        DevicePopup.setTabOrder(self.alpacaHostAddress, self.alpacaPort)
        DevicePopup.setTabOrder(self.alpacaPort, self.alpacaDeviceList)
        DevicePopup.setTabOrder(self.alpacaDeviceList, self.alpacaDiscover)
        DevicePopup.setTabOrder(self.alpacaDiscover, self.alpacaUser)
        DevicePopup.setTabOrder(self.alpacaUser, self.alpacaPassword)
        DevicePopup.setTabOrder(self.alpacaPassword, self.alpacaCopyConfig)
        DevicePopup.setTabOrder(self.alpacaCopyConfig, self.ascomDevice)
        DevicePopup.setTabOrder(self.ascomDevice, self.ascomSelector)
        DevicePopup.setTabOrder(self.ascomSelector, self.indiHostAddress)
        DevicePopup.setTabOrder(self.indiHostAddress, self.indiPort)
        DevicePopup.setTabOrder(self.indiPort, self.indiDeviceList)
        DevicePopup.setTabOrder(self.indiDeviceList, self.indiDiscover)
        DevicePopup.setTabOrder(self.indiDiscover, self.indiMessages)
        DevicePopup.setTabOrder(self.indiMessages, self.indiLoadConfig)
        DevicePopup.setTabOrder(self.indiLoadConfig, self.indiCopyConfig)
        DevicePopup.setTabOrder(self.indiCopyConfig, self.selectAstrometryAppPath)
        DevicePopup.setTabOrder(self.selectAstrometryAppPath, self.astrometryAppPath)
        DevicePopup.setTabOrder(self.astrometryAppPath, self.selectAstrometryIndexPath)
        DevicePopup.setTabOrder(self.selectAstrometryIndexPath, self.astrometryIndexPath)
        DevicePopup.setTabOrder(self.astrometryIndexPath, self.astrometryDeviceList)
        DevicePopup.setTabOrder(self.astrometryDeviceList, self.astrometrySearchRadius)
        DevicePopup.setTabOrder(self.astrometrySearchRadius, self.astrometryTimeout)
        DevicePopup.setTabOrder(self.astrometryTimeout, self.selectAstapAppPath)
        DevicePopup.setTabOrder(self.selectAstapAppPath, self.astapAppPath)
        DevicePopup.setTabOrder(self.astapAppPath, self.selectAstapIndexPath)
        DevicePopup.setTabOrder(self.selectAstapIndexPath, self.astapIndexPath)
        DevicePopup.setTabOrder(self.astapIndexPath, self.astapDeviceList)
        DevicePopup.setTabOrder(self.astapDeviceList, self.astapSearchRadius)
        DevicePopup.setTabOrder(self.astapSearchRadius, self.astapTimeout)
        DevicePopup.setTabOrder(self.astapTimeout, self.onlineWeatherHostAddress)
        DevicePopup.setTabOrder(self.onlineWeatherHostAddress, self.onlineWeatherApiKey)
        DevicePopup.setTabOrder(self.onlineWeatherApiKey, self.relayHostAddress)
        DevicePopup.setTabOrder(self.relayHostAddress, self.relayUser)
        DevicePopup.setTabOrder(self.relayUser, self.relayPassword)
        DevicePopup.setTabOrder(self.relayPassword, self.cancel)
        DevicePopup.setTabOrder(self.cancel, self.ok)
        DevicePopup.setTabOrder(self.ok, self.tab)

    def retranslateUi(self, DevicePopup):
        _translate = QtCore.QCoreApplication.translate
        DevicePopup.setWindowTitle(_translate("DevicePopup", "Device Setup"))
        self.label_mountIP_7.setText(_translate("DevicePopup", "Device Name"))
        self.alpacaPort.setToolTip(_translate("DevicePopup", "Port of ALPACA server"))
        self.alpacaPort.setText(_translate("DevicePopup", "11111"))
        self.label_mountIP_8.setText(_translate("DevicePopup", "Host Address"))
        self.label_mountIP_10.setText(_translate("DevicePopup", "Host Port"))
        self.alpacaHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address of INDI server, ip address and host names!</p></body></html>"))
        self.alpacaHostAddress.setText(_translate("DevicePopup", "localhost"))
        self.label_mountIP_11.setText(_translate("DevicePopup", "User"))
        self.label_mountIP_12.setText(_translate("DevicePopup", "Password"))
        self.alpacaUser.setToolTip(_translate("DevicePopup", "User for alpaca device"))
        self.alpacaPassword.setToolTip(_translate("DevicePopup", "Passwor for alpaca device"))
        self.alpacaCopyConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.alpacaCopyConfig.setText(_translate("DevicePopup", "Copy config to all devices"))
        self.alpacaDiscover.setToolTip(_translate("DevicePopup", "Search does list name of INDI devices of this certain type"))
        self.alpacaDiscover.setText(_translate("DevicePopup", "Search"))
        self.tab.setTabText(self.tab.indexOf(self.ALPACA), _translate("DevicePopup", "ALPACA"))
        self.ascomSelector.setToolTip(_translate("DevicePopup", "Starting the ASCOM device chooser."))
        self.ascomSelector.setText(_translate("DevicePopup", "ASCOM device setup"))
        self.label_mountIP_15.setText(_translate("DevicePopup", "Device Name"))
        self.ascomDevice.setToolTip(_translate("DevicePopup", "Drvice name of ASCOM driver."))
        self.tab.setTabText(self.tab.indexOf(self.ASCOM), _translate("DevicePopup", "ASCOM"))
        self.label_mountIP_3.setText(_translate("DevicePopup", "Host Address"))
        self.indiHostAddress.setToolTip(_translate("DevicePopup", "IP address of INDI server, ip address and host names!"))
        self.indiHostAddress.setText(_translate("DevicePopup", "localhost"))
        self.label_mountIP_4.setText(_translate("DevicePopup", "Host Port"))
        self.indiDiscover.setToolTip(_translate("DevicePopup", "Search does list name of INDI devices of this certain type"))
        self.indiDiscover.setText(_translate("DevicePopup", "Search"))
        self.label_mountIP_5.setText(_translate("DevicePopup", "Device Name"))
        self.indiMessages.setToolTip(_translate("DevicePopup", "When checked, the indi messages received will be shown inthe central message window"))
        self.indiMessages.setText(_translate("DevicePopup", "Show Messages"))
        self.indiPort.setToolTip(_translate("DevicePopup", "Port of INDI server"))
        self.indiPort.setText(_translate("DevicePopup", "7624"))
        self.indiLoadConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.indiLoadConfig.setText(_translate("DevicePopup", "Load configuration on start"))
        self.indiCopyConfig.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Checked if you would like to enable blind solver function in SGPro</p></body></html>"))
        self.indiCopyConfig.setText(_translate("DevicePopup", "Copy config to all devices"))
        self.tab.setTabText(self.tab.indexOf(self.INDI), _translate("DevicePopup", "INDI / INDIGO"))
        self.label_mountIP_14.setText(_translate("DevicePopup", "Folder astrometry index"))
        self.astrometryIndexPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astrometry.net index files. Where native installation or e.g. KStars put the files.</p></body></html>"))
        self.astrometryIndexPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstrometryIndexPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstrometryIndexPath.setText(_translate("DevicePopup", "Select"))
        self.astrometryDeviceList.setToolTip(_translate("DevicePopup", "Select the device."))
        self.label_mountIP_6.setText(_translate("DevicePopup", "Device Name"))
        self.label_316.setText(_translate("DevicePopup", "Timeout"))
        self.astrometrySearchRadius.setToolTip(_translate("DevicePopup", "Set the search radius for the plate solver. "))
        self.astrometryTimeout.setToolTip(_translate("DevicePopup", "Set the timeout for the plate solver."))
        self.label_315.setText(_translate("DevicePopup", "Search radius"))
        self.label_317.setText(_translate("DevicePopup", "s"))
        self.label_318.setText(_translate("DevicePopup", "deg FOV"))
        self.label_mountIP_17.setText(_translate("DevicePopup", "Folder application"))
        self.astrometryAppPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astrometry.net binary files. Where native installation or e.g. KStars put the files.</p></body></html>"))
        self.astrometryAppPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstrometryAppPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstrometryAppPath.setText(_translate("DevicePopup", "Select"))
        self.tab.setTabText(self.tab.indexOf(self.ASTROMETRY), _translate("DevicePopup", "ASTROMETRY"))
        self.label_mountIP_16.setText(_translate("DevicePopup", "Device Name"))
        self.astapTimeout.setToolTip(_translate("DevicePopup", "Set the timeout for the plate solver."))
        self.astapSearchRadius.setToolTip(_translate("DevicePopup", "Set the search radius for the plate solver. "))
        self.label_319.setText(_translate("DevicePopup", "Timeout"))
        self.label_320.setText(_translate("DevicePopup", "Search radius"))
        self.label_321.setText(_translate("DevicePopup", "s"))
        self.label_322.setText(_translate("DevicePopup", "deg FOV"))
        self.astapIndexPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astap star database.</p></body></html>"))
        self.astapIndexPath.setText(_translate("DevicePopup", "/usr/"))
        self.astapAppPath.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Path to astap application.</p></body></html>"))
        self.astapAppPath.setText(_translate("DevicePopup", "/usr/"))
        self.selectAstapIndexPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstapIndexPath.setText(_translate("DevicePopup", "Select"))
        self.selectAstapAppPath.setToolTip(_translate("DevicePopup", "Select the directory of the astrometry.net index files"))
        self.selectAstapAppPath.setText(_translate("DevicePopup", "Select"))
        self.astapDeviceList.setToolTip(_translate("DevicePopup", "Select the device."))
        self.label_mountIP_18.setText(_translate("DevicePopup", "Folder astrometry index"))
        self.label_mountIP_19.setText(_translate("DevicePopup", "Folder application"))
        self.tab.setTabText(self.tab.indexOf(self.ASTAP), _translate("DevicePopup", "ASTAP"))
        self.label_mountIP_33.setText(_translate("DevicePopup", "Host Address"))
        self.onlineWeatherHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address for open weather service</p></body></html>"))
        self.onlineWeatherHostAddress.setText(_translate("DevicePopup", "api.openweathermap.org"))
        self.onlineWeatherApiKey.setToolTip(_translate("DevicePopup", "<html><head/><body><p>api key for open weather service</p></body></html>"))
        self.label_mountIP_34.setText(_translate("DevicePopup", "API Key"))
        self.tab.setTabText(self.tab.indexOf(self.weather), _translate("DevicePopup", "Online Weather"))
        self.label_mountIP_35.setText(_translate("DevicePopup", "Host Address"))
        self.relayHostAddress.setToolTip(_translate("DevicePopup", "<html><head/><body><p>IP address for relay device</p></body></html>"))
        self.relayPassword.setToolTip(_translate("DevicePopup", "<html><head/><body><p>Passwor for relay device</p></body></html>"))
        self.relayUser.setToolTip(_translate("DevicePopup", "<html><head/><body><p>User for relay device</p></body></html>"))
        self.label_mountIP_36.setText(_translate("DevicePopup", "User"))
        self.label_mountIP_37.setText(_translate("DevicePopup", "Password"))
        self.tab.setTabText(self.tab.indexOf(self.relay), _translate("DevicePopup", "Relay"))
        self.ok.setText(_translate("DevicePopup", "OK"))
        self.cancel.setText(_translate("DevicePopup", "Cancel"))
        self.iconPixmap.setText(_translate("DevicePopup", "TextLabel"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    DevicePopup = QtWidgets.QWidget()
    ui = Ui_DevicePopup()
    ui.setupUi(DevicePopup)
    DevicePopup.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# written in python3, (c) 2019-2021 by mworion
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import datetime
from dateutil.tz import tzutc

# external packages

# local imports


def timeTz(*args):
    return datetime.datetime.now(tzutc()).timetuple()


def addLoggingLevel(levelName, levelNum, methodName=None):
    """
    Comprehensively adds a new logging level to the `logging` module and the
    currently configured logging class.

    `levelName` becomes an attribute of the `logging` module with the value
    `levelNum`.

    `methodName` becomes a convenience method for both `logging`
    itself and the class returned by `logging.getLoggerClass()` (usually just
    `logging.Logger`).

    If `methodName` is not specified, `levelName.lower()` is
    used.

    To avoid accidental clobberings of existing attributes, this method will
    raise an `AttributeError` if the level name is already an attribute of the
    `logging` module or if the method name is already present

    This method was inspired by the answers to Stack Overflow post
    http://stackoverflow.com/q/2183233/2988730, especially
    http://stackoverflow.com/a/13638084/2988730

    Example
    -------
    >>> addLoggingLevel('TRACE', logging.DEBUG - 5)
    >>> logging.getLogger(__name__).setLevel("TRACE")
    >>> logging.getLogger(__name__).trace('that worked')
    >>> logging.trace('so did this')
    >>> logging.TRACE
    5

    """
    if not methodName:
        methodName = levelName.lower()

    if hasattr(logging, levelName):
        return

    if hasattr(logging, methodName):
        return

    if hasattr(logging.getLoggerClass(), methodName):
        return

    def logForLevel(self, message, *args, **kwargs):
        if self.isEnabledFor(levelNum):
            self._log(levelNum, message, args, **kwargs)

    def logToRoot(message, *args, **kwargs):
        if logging.root.isEnabledFor(levelNum):
            logging.root._log(levelNum, message, args, **kwargs)

    logging.addLevelName(levelNum, levelName)
    setattr(logging, levelName, levelNum)
    setattr(logging.getLoggerClass(), methodName, logForLevel)
    setattr(logging, methodName, logToRoot)


def setupLogging():
    """
    setupLogging defines the logger and formats and disables unnecessary library logging

    :return: true for test purpose
    """
    addLoggingLevel('HEADER', 55)
    addLoggingLevel('UI', 35)
    addLoggingLevel('TRACE', 5)

    logging.Formatter.converter = timeTz
    timeTag = datetime.datetime.now(datetime.timezone.utc).strftime('%Y-%m-%d')
    name = f'mw4-{timeTag}.log'
    logging.basicConfig(level=logging.DEBUG,
                        format='[%(asctime)s.%(msecs)03d]'
                               '[%(levelname)1.1s]'
                               '[%(filename)15.15s]'
                               '[%(lineno)4s]'
                               ' %(message)s',
                        handlers=[logging.FileHandler(name)],
                        datefmt='%Y-%m-%d %H:%M:%S',
                        )

    # setting different log level for imported packages to avoid unnecessary data
    # urllib3 is used by requests, so we have to add this as well
    logging.getLogger('PyQt5').setLevel(logging.WARNING)
    logging.getLogger('requests').setLevel(logging.WARNING)
    logging.getLogger('urllib3').setLevel(logging.WARNING)
    logging.getLogger('matplotlib').setLevel(logging.WARNING)
    # logging.getLogger('astropy').setLevel(logging.WARNING)
    return True


def setCustomLoggingLevel(level='WARN'):
    """
    Setting the log level according to the setting in the gui.

    :return: true for test purpose
    """
    logging.getLogger().setLevel(level)

    return True
